@@ -49,44 +49,52 @@
 #include "nsHTMLTokens.h"
 #include "nsIURI.h"
 #include "nsTextFragment.h"
+#ifdef MOZ_XUL
 #include "nsXULElement.h"
+#endif
 #include "nsXULAtoms.h"
 #include "nsXBLProtoImplProperty.h"
 #include "nsXBLProtoImplMethod.h"
 #include "nsXBLProtoImplField.h"
+#include "nsXBLPrototypeBinding.h"
+#include "nsIConsoleService.h"
+#include "nsIScriptError.h"
+#include "nsIStringBundle.h"
 
-static NS_DEFINE_CID(kCSSParserCID,              NS_CSSPARSER_CID);
-
+#include "nsIPrincipal.h"
 nsresult
 NS_NewXBLContentSink(nsIXMLContentSink** aResult,
                      nsIDocument* aDoc,
-                     nsIURI* aURL,
-                     nsIWebShell* aWebShell)
+                     nsIURI* aURI,
+                     nsISupports* aContainer)
 {
-  NS_PRECONDITION(nsnull != aResult, "null ptr");
-  if (!aResult)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(aResult);
+
   nsXBLContentSink* it;
   NS_NEWXPCOM(it, nsXBLContentSink);
-  if (!it)
-    return NS_ERROR_OUT_OF_MEMORY;
-  nsresult rv = it->Init(aDoc, aURL, aWebShell);
-  if (NS_FAILED(rv))
-    return rv;
-  return it->QueryInterface(NS_GET_IID(nsIXMLContentSink), (void **)aResult);
+  NS_ENSURE_TRUE(it, NS_ERROR_OUT_OF_MEMORY);
+
+  nsCOMPtr<nsIXMLContentSink> kungFuDeathGrip = it;
+  nsresult rv = it->Init(aDoc, aURI, aContainer);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return CallQueryInterface(it, aResult);
 }
 
 nsXBLContentSink::nsXBLContentSink()
+  : mState(eXBL_InDocument),
+    mSecondaryState(eXBL_None),
+    mDocInfo(nsnull),
+    mIsChromeOrResource(PR_FALSE),
+    mBinding(nsnull),
+    mHandler(nsnull),
+    mImplementation(nsnull),
+    mImplMember(nsnull),
+    mProperty(nsnull),
+    mMethod(nsnull),
+    mField(nsnull)
 {
-  mState = eXBL_InDocument;
-  mSecondaryState = eXBL_None;
-  mDocInfo = nsnull;
-  mIsChromeOrResource = PR_FALSE;
-  mImplementation = nsnull;
-  mImplMember = nsnull;
-  mProperty = nsnull;
-  mMethod = nsnull;
-  mField = nsnull;
+  mPrettyPrintXML = PR_FALSE;
 }
 
 nsXBLContentSink::~nsXBLContentSink()
@@ -95,11 +103,11 @@ nsXBLContentSink::~nsXBLContentSink()
 
 nsresult
 nsXBLContentSink::Init(nsIDocument* aDoc,
-                       nsIURI* aURL,
-                       nsIWebShell* aContainer)
+                       nsIURI* aURI,
+                       nsISupports* aContainer)
 {
   nsresult rv;
-  rv = nsXMLContentSink::Init(aDoc, aURL, aContainer);
+  rv = nsXMLContentSink::Init(aDoc, aURI, aContainer, nsnull);
   return rv;
 }
 
@@ -131,11 +139,11 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
       // BindingAttached and BindingDetached, and they're still implemented
       // using handlers.  At some point, we need to change these to just
       // be special functions on the class instead.
-      nsCOMPtr<nsIXBLPrototypeHandler> handler;
+      nsXBLPrototypeHandler* handler;
       if (mSecondaryState == eXBL_InConstructor)
-        mBinding->GetConstructor(getter_AddRefs(handler));
+        handler = mBinding->GetConstructor();
       else
-        mBinding->GetDestructor(getter_AddRefs(handler));
+        handler = mBinding->GetDestructor();
 
       // Get the text and add it to the constructor/destructor.
       handler->AppendHandlerText(text);
@@ -150,7 +158,8 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
     }
     else if (mSecondaryState == eXBL_InBody) {
       // Get the text and add it to the method
-      mMethod->AppendBodyText(text);
+      if (mMethod)
+        mMethod->AppendBodyText(text);
     }
     else if (mSecondaryState == eXBL_InField) {
       // Get the text and add it to the method
@@ -162,43 +171,141 @@ nsXBLContentSink::FlushText(PRBool aCreateTextNode,
     return NS_OK;
   }
 
-  PRBool isWS = PR_TRUE;
-  if (mTextLength > 0) {
-    const PRUnichar* cp = mText;
-    const PRUnichar* end = mText + mTextLength;
-    while (cp < end) {
-      PRUnichar ch = *cp++;
-      if (!XP_IS_SPACE(ch)) {
-        isWS = PR_FALSE;
-        break;
+  nsIContent* content = GetCurrentContent();
+  if (content && (content->GetNodeInfo()->NamespaceEquals(kNameSpaceID_XBL) || (
+      content->GetNodeInfo()->NamespaceEquals(kNameSpaceID_XUL) &&
+      content->Tag() != nsXULAtoms::label &&
+      content->Tag() != nsXULAtoms::description))) {
+
+    PRBool isWS = PR_TRUE;
+    if (mTextLength > 0) {
+      const PRUnichar* cp = mText;
+      const PRUnichar* end = mText + mTextLength;
+      while (cp < end) {
+        PRUnichar ch = *cp++;
+        if (!XP_IS_SPACE(ch)) {
+          isWS = PR_FALSE;
+          break;
+        }
       }
     }
+
+    if (isWS && mTextLength > 0) {
+      mTextLength = 0;
+      if (aDidFlush)
+        *aDidFlush = PR_TRUE;
+      return NS_OK;
+    }
   }
 
-  if (isWS && mTextLength > 0) {
-    mTextLength = 0;
-    if (aDidFlush)
-      *aDidFlush = PR_TRUE;
-    return NS_OK;
+  return nsXMLContentSink::FlushText(aCreateTextNode, aDidFlush);
+}
+
+NS_IMETHODIMP
+nsXBLContentSink::ReportError(const PRUnichar* aErrorText, 
+                              const PRUnichar* aSourceText)
+{
+  // XXX We should make sure the binding has no effect, but that it also
+  // gets destroyed properly without leaking.  Perhaps we should even
+  // ensure that the content that was bound is displayed with no
+  // binding.
+
+  // Report the error to the error console.
+  nsCOMPtr<nsIConsoleService> consoleService =
+    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
+  if (consoleService) {
+    // XXX It would be nice if the parser didn't pre-format it for us,
+    // because then we could create a instance of nsIScriptError and the
+    // error console would format this much more nicely for us.
+    // However, that would require duplicating even more code between
+    // the XML content sink and the XUL content sink.
+
+    consoleService->LogStringMessage(aErrorText);
   }
 
-  return nsXMLContentSink::FlushText(aCreateTextNode, aDidFlush);
+#ifdef DEBUG
+  // Report the error to stderr.
+  fprintf(stderr,
+          "\n%s\n%s\n\n",
+          NS_LossyConvertUCS2toASCII(aErrorText).get(),
+          NS_LossyConvertUCS2toASCII(aSourceText).get());
+#endif
+
+  // Most of what this does won't be too useful, but whatever...
+  return nsXMLContentSink::ReportError(aErrorText, aSourceText);
+}
+
+nsresult
+nsXBLContentSink::ReportUnexpectedElement(nsIAtom* aElementName,
+                                          PRUint32 aLineNumber)
+{
+  // XXX we should really somehow stop the parse and drop the binding
+  // instead of just letting the XML sink build the content model like
+  // we do...
+  mState = eXBL_Error;
+  nsAutoString elementName;
+  aElementName->ToString(elementName);
+
+  nsresult rv;
+  nsCOMPtr<nsIConsoleService> consoleService =
+    do_GetService(NS_CONSOLESERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIScriptError> errorObject =
+    do_CreateInstance(NS_SCRIPTERROR_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIStringBundleService> stringBundleService =
+    do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIStringBundle> bundle;
+  rv = stringBundleService->CreateBundle(
+           "chrome://global/locale/xbl.properties", getter_AddRefs(bundle));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  const PRUnichar* params[] = { elementName.get() };
+  
+  nsXPIDLString errorText;
+  rv = bundle->FormatStringFromName(NS_LITERAL_STRING("UnexpectedElement").get(),
+                                    params, NS_ARRAY_LENGTH(params),
+                                    getter_Copies(errorText));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCAutoString documentURI;
+  mDocumentURI->GetSpec(documentURI);
+  
+  rv = errorObject->Init(errorText.get(),
+                         NS_ConvertUTF8toUCS2(documentURI).get(),
+                         EmptyString().get(), /* source line */
+                         aLineNumber,
+                         0,  /* column number */
+                         nsIScriptError::errorFlag,
+                         "XBL Content Sink");
+
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  return consoleService->LogMessage(errorObject);
 }
 
 NS_IMETHODIMP 
 nsXBLContentSink::HandleStartElement(const PRUnichar *aName, 
                                      const PRUnichar **aAtts, 
                                      PRUint32 aAttsCount, 
-                                     PRUint32 aIndex, 
+                                     PRInt32 aIndex, 
                                      PRUint32 aLineNumber)
 {
   nsresult rv = nsXMLContentSink::HandleStartElement(aName,aAtts,aAttsCount,aIndex,aLineNumber);
   if (NS_FAILED(rv))
     return rv;
 
-  if (mState == eXBL_InBinding && !mBinding)
+  if (mState == eXBL_InBinding && !mBinding) {
+    // XXX need to return nsresult here.  Need error-handling in this
+    // file in general.
     ConstructBinding();
-  
+    if (!mBinding) {
+      return NS_ERROR_UNEXPECTED;
+    }
+  }
+
   return rv;
 }
 
@@ -255,15 +362,16 @@ nsXBLContentSink::HandleEndElement(const PRUnichar *aName)
           mSecondaryState = eXBL_InMethod;
         return NS_OK;
       }
-
+      else if (mState == eXBL_InBindings && tagAtom == nsXBLAtoms::bindings) {
+        mState = eXBL_InDocument;
+      }
+      
       nsresult rv = nsXMLContentSink::HandleEndElement(aName);
       if (NS_FAILED(rv))
         return rv;
 
-      if (mState == eXBL_InImplementation && tagAtom == nsXBLAtoms::implementation)
-        mState = eXBL_InBinding;
-      else if (mState == eXBL_InBinding && tagAtom == nsXBLAtoms::binding) {
-        mState = eXBL_InDocument;
+      if (mState == eXBL_InBinding && tagAtom == nsXBLAtoms::binding) {
+        mState = eXBL_InBindings;
         mBinding->Initialize();
         mBinding = nsnull; // Clear our current binding ref.
       }
@@ -288,44 +396,71 @@ PRBool
 nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts, 
                                   PRUint32 aAttsCount, 
                                   PRInt32 aNameSpaceID, 
-                                  nsIAtom* aTagName)
+                                  nsIAtom* aTagName,
+                                  PRUint32 aLineNumber)
 {
+  if (mState == eXBL_Error) {
+    return PR_TRUE;
+  }
+  
   PRBool ret = PR_TRUE;
   if (aNameSpaceID == kNameSpaceID_XBL) {
     if (aTagName == nsXBLAtoms::bindings) {
+      if (mState != eXBL_InDocument) {
+        ReportUnexpectedElement(aTagName, aLineNumber);
+        return PR_TRUE;
+      }
+      
       NS_NewXBLDocumentInfo(mDocument, &mDocInfo);
-      if (!mDocInfo)
-        return NS_ERROR_FAILURE;
+      if (!mDocInfo) {
+        mState = eXBL_Error;
+        return PR_TRUE;
+      }
 
-      nsCOMPtr<nsIBindingManager> bindingManager;
-      mDocument->GetBindingManager(getter_AddRefs(bindingManager));
-      bindingManager->PutXBLDocumentInfo(mDocInfo);
+      mDocument->GetBindingManager()->PutXBLDocumentInfo(mDocInfo);
 
-      nsCOMPtr<nsIURI> url;
-      mDocument->GetDocumentURL(getter_AddRefs(url));
+      nsIURI *uri = mDocument->GetDocumentURI();
       
       PRBool isChrome = PR_FALSE;
       PRBool isRes = PR_FALSE;
 
-      url->SchemeIs("chrome", &isChrome);
-      url->SchemeIs("resource", &isRes);
+      uri->SchemeIs("chrome", &isChrome);
+      uri->SchemeIs("resource", &isRes);
       mIsChromeOrResource = isChrome || isRes;
       
       nsIXBLDocumentInfo* info = mDocInfo;
       NS_RELEASE(info); // We keep a weak ref. We've created a cycle between doc/binding manager/doc info.
+      mState = eXBL_InBindings; 
     }
-    else if (aTagName == nsXBLAtoms::binding)
+    else if (aTagName == nsXBLAtoms::binding) {
+      if (mState != eXBL_InBindings) {
+        ReportUnexpectedElement(aTagName, aLineNumber);
+        return PR_TRUE;
+      }
       mState = eXBL_InBinding;
+    }
     else if (aTagName == nsXBLAtoms::handlers) {
+      if (mState != eXBL_InBinding) {
+        ReportUnexpectedElement(aTagName, aLineNumber);
+        return PR_TRUE;
+      }
       mState = eXBL_InHandlers;
       ret = PR_FALSE; // The XML content sink should not do anything with <handlers>.
     }
     else if (aTagName == nsXBLAtoms::handler) {
+      if (mState != eXBL_InHandlers) {
+        ReportUnexpectedElement(aTagName, aLineNumber);
+        return PR_TRUE;
+      }
       mSecondaryState = eXBL_InHandler;
-      ConstructHandler(aAtts);
+      ConstructHandler(aAtts, aLineNumber);
       ret = PR_FALSE;
     }
     else if (aTagName == nsXBLAtoms::resources) {
+      if (mState != eXBL_InBinding) {
+        ReportUnexpectedElement(aTagName, aLineNumber);
+        return PR_TRUE;
+      }
       mState = eXBL_InResources;
       ret = PR_FALSE; // The XML content sink should ignore all <resources>.
     }
@@ -335,6 +470,10 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
       ret = PR_FALSE; // The XML content sink should ignore everything within a <resources> block.
     }
     else if (aTagName == nsXBLAtoms::implementation) {
+      if (mState != eXBL_InBinding) {
+        ReportUnexpectedElement(aTagName, aLineNumber);
+        return PR_TRUE;
+      }
       mState = eXBL_InImplementation;
       ConstructImplementation(aAtts);
       ret = PR_FALSE; // The XML content sink should ignore the <implementation>.
@@ -342,42 +481,57 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
     else if (mState == eXBL_InImplementation) {
       if (aTagName == nsXBLAtoms::constructor) {
         mSecondaryState = eXBL_InConstructor;
-        nsCOMPtr<nsIXBLPrototypeHandler> newHandler;
-        NS_NewXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull,
-                                  nsnull, nsnull, nsnull, nsnull, nsnull,
-                                  getter_AddRefs(newHandler));
+        nsXBLPrototypeHandler* newHandler;
+        newHandler = new nsXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull, mBinding);
         newHandler->SetEventName(nsXBLAtoms::constructor);
+        newHandler->SetLineNumber(aLineNumber);
         mBinding->SetConstructor(newHandler);
       }
       else if (aTagName == nsXBLAtoms::destructor) {
         mSecondaryState = eXBL_InDestructor;
-        nsCOMPtr<nsIXBLPrototypeHandler> newHandler;
-        NS_NewXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull,
-                                  nsnull, nsnull, nsnull, nsnull, nsnull,
-                                  getter_AddRefs(newHandler));
+        nsXBLPrototypeHandler* newHandler;
+        newHandler = new nsXBLPrototypeHandler(nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull, nsnull, nsnull,
+                                               nsnull, nsnull, mBinding);
         newHandler->SetEventName(nsXBLAtoms::destructor);
+        newHandler->SetLineNumber(aLineNumber);
         mBinding->SetDestructor(newHandler);
       }
       else if (aTagName == nsXBLAtoms::field) {
         mSecondaryState = eXBL_InField;
-        ConstructField(aAtts);
+        ConstructField(aAtts, aLineNumber);
       }
       else if (aTagName == nsXBLAtoms::property) {
         mSecondaryState = eXBL_InProperty;
         ConstructProperty(aAtts);
       }
-      else if (aTagName == nsXBLAtoms::getter)
+      else if (aTagName == nsXBLAtoms::getter) {
+        if (mSecondaryState == eXBL_InProperty && mProperty) {
+          mProperty->SetGetterLineNumber(aLineNumber);
+        }        
         mSecondaryState = eXBL_InGetter;
-      else if (aTagName == nsXBLAtoms::setter)
+      }
+      else if (aTagName == nsXBLAtoms::setter) {
+        if (mSecondaryState == eXBL_InProperty && mProperty) {
+          mProperty->SetSetterLineNumber(aLineNumber);
+        }
         mSecondaryState = eXBL_InSetter;
+      }
       else if (aTagName == nsXBLAtoms::method) {
         mSecondaryState = eXBL_InMethod;
         ConstructMethod(aAtts);
       }
       else if (aTagName == nsXBLAtoms::parameter)
         ConstructParameter(aAtts);
-      else if (aTagName == nsXBLAtoms::body)
+      else if (aTagName == nsXBLAtoms::body) {
+        if (mSecondaryState == eXBL_InMethod && mMethod) {
+          // stash away the line number
+          mMethod->SetLineNumber(aLineNumber);
+        }
         mSecondaryState = eXBL_InBody;
+      }
 
       ret = PR_FALSE; // Ignore everything we encounter inside an <implementation> block.
     }
@@ -389,33 +543,41 @@ nsXBLContentSink::OnOpenContainer(const PRUnichar **aAtts,
 void 
 nsXBLContentSink::ConstructBinding()
 {
-  nsCOMPtr<nsIContent> binding = getter_AddRefs(GetCurrentContent());
+  nsCOMPtr<nsIContent> binding = GetCurrentContent();
   nsAutoString id;
   binding->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id, id);
   nsCAutoString cid; cid.AssignWithConversion(id);
 
   if (!cid.IsEmpty()) {
-    NS_NewXBLPrototypeBinding(cid, binding, mDocInfo, getter_AddRefs(mBinding));
-    mDocInfo->SetPrototypeBinding(cid, mBinding);
-    binding->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::id, PR_FALSE);
+    mBinding = new nsXBLPrototypeBinding();
+    if (mBinding) {
+      if (NS_SUCCEEDED(mBinding->Init(cid, mDocInfo, binding))) {
+        mDocInfo->SetPrototypeBinding(cid, mBinding);
+        binding->UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::id, PR_FALSE);
+      } else {
+        delete mBinding;
+        mBinding = nsnull;
+      }
+    }
   }
 }
 
 
 void
-nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts)
+nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts, PRUint32 aLineNumber)
 {
   nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
 
-  const PRUnichar* event      = nsnull;
-  const PRUnichar* modifiers  = nsnull;
-  const PRUnichar* button     = nsnull;
-  const PRUnichar* clickcount = nsnull;
-  const PRUnichar* keycode    = nsnull;
-  const PRUnichar* charcode   = nsnull;
-  const PRUnichar* phase      = nsnull;
-  const PRUnichar* command    = nsnull;
-  const PRUnichar* action     = nsnull;
+  const PRUnichar* event          = nsnull;
+  const PRUnichar* modifiers      = nsnull;
+  const PRUnichar* button         = nsnull;
+  const PRUnichar* clickcount     = nsnull;
+  const PRUnichar* keycode        = nsnull;
+  const PRUnichar* charcode       = nsnull;
+  const PRUnichar* phase          = nsnull;
+  const PRUnichar* command        = nsnull;
+  const PRUnichar* action         = nsnull;
+  const PRUnichar* preventdefault = nsnull;
 
   for (; *aAtts; aAtts += 2) {
     // Get upper-cased key
@@ -446,6 +608,8 @@ nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts)
       command = aAtts[1];
     else if (nameAtom == nsXBLAtoms::action)
       action = aAtts[1];
+    else if (nameAtom == nsXBLAtoms::preventdefault)
+      preventdefault = aAtts[1];
     else {
        // Nope, it's some irrelevant attribute. Ignore it and move on.
      }
@@ -458,11 +622,14 @@ nsXBLContentSink::ConstructHandler(const PRUnichar **aAtts)
 
   // All of our pointers are now filled in.  Construct our handler with all of these
   // parameters.
-  nsCOMPtr<nsIXBLPrototypeHandler> newHandler;
-  NS_NewXBLPrototypeHandler(event, phase, action, command, 
-                            keycode, charcode, modifiers, button,
-                            clickcount, getter_AddRefs(newHandler));
+  nsXBLPrototypeHandler* newHandler;
+  newHandler = new nsXBLPrototypeHandler(event, phase, action, command,
+                                         keycode, charcode, modifiers, button,
+                                         clickcount, preventdefault, mBinding);
+
   if (newHandler) {
+    newHandler->SetLineNumber(aLineNumber);
+    
     // Add this handler to our chain of handlers.
     if (mHandler)
       mHandler->SetNextHandler(newHandler); // Already have a chain. Just append to the end.
@@ -527,7 +694,25 @@ nsXBLContentSink::ConstructImplementation(const PRUnichar **aAtts)
       name = aAtts[1];
     }
     else if (nameAtom == nsXBLAtoms::implements) {
-      mBinding->ConstructInterfaceTable(nsDependentString(aAtts[1]));
+      // Only allow implementation of interfaces via XBL if the principal of
+      // our XBL document has UniversalXPConnect privileges.  No principal
+      // means no privs!
+      
+      nsIPrincipal* principal = mDocument->GetPrincipal();
+      if (principal) {
+        // XXX this api is so badly tied to JS it's not even funny.  We don't
+        // have a concept of enabling capabilities on a per-principal basis,
+        // but only on a per-principal-and-JS-stackframe basis!  So for now
+        // this is basically equivalent to testing that we have the system
+        // principal, since there is no JS stackframe in sight here...
+        PRBool hasUniversalXPConnect;
+        nsresult rv = principal->IsCapabilityEnabled("UniversalXPConnect",
+                                                     nsnull,
+                                                     &hasUniversalXPConnect);
+        if (NS_SUCCEEDED(rv) && hasUniversalXPConnect) {
+          mBinding->ConstructInterfaceTable(nsDependentString(aAtts[1]));
+        }
+      }
     }
   }
 
@@ -535,7 +720,7 @@ nsXBLContentSink::ConstructImplementation(const PRUnichar **aAtts)
 }
 
 void
-nsXBLContentSink::ConstructField(const PRUnichar **aAtts)
+nsXBLContentSink::ConstructField(const PRUnichar **aAtts, PRUint32 aLineNumber)
 {
   nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
 
@@ -563,6 +748,8 @@ nsXBLContentSink::ConstructField(const PRUnichar **aAtts)
   // parameters.
   mField = new nsXBLProtoImplField(name, readonly);
   if (mField) {
+    mField->SetLineNumber(aLineNumber);
+    
     // Add this member to our chain.
     if (mImplMember)
       mImplMember->SetNext(mField); // Already have a chain. Just append to the end.
@@ -681,48 +868,51 @@ nsXBLContentSink::ConstructParameter(const PRUnichar **aAtts)
 }
 
 nsresult
-nsXBLContentSink::CreateElement(const PRUnichar** aAtts, 
-                                PRUint32 aAttsCount, 
-                                PRInt32 aNameSpaceID, 
-                                nsINodeInfo* aNodeInfo, 
-                                nsIContent** aResult)
+nsXBLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
+                                nsINodeInfo* aNodeInfo, PRUint32 aLineNumber,
+                                nsIContent** aResult, PRBool* aAppendContent)
 {
-  if (aNameSpaceID == nsXULAtoms::nameSpaceID) {
-    nsXULPrototypeElement* prototype = new nsXULPrototypeElement(0);
-    if (!prototype)
-      return NS_ERROR_OUT_OF_MEMORY;
-    
-    prototype->mNodeInfo = aNodeInfo;
-
-    // Reset the refcnt to 0.  Normally XUL prototype elements get a refcnt of 1
-    // to represent ownership by the XUL prototype document.  In our case we have
-    // no prototype document, and our initial ref count of 1 will come from being
-    // wrapped by a real XUL element in the Create call below.
-    prototype->mRefCnt = 0;
-
-    AddAttributesToXULPrototype(aAtts, aAttsCount, prototype);
-
-    // Following this function call, the prototype's ref count will be 1.
-    nsresult rv = nsXULElement::Create(prototype, mDocument, PR_FALSE, aResult);
-
-    if (NS_FAILED(rv)) return rv;
-    return NS_OK;
+#ifdef MOZ_XUL
+  if (!aNodeInfo->NamespaceEquals(kNameSpaceID_XUL)) {
+#endif
+    return nsXMLContentSink::CreateElement(aAtts, aAttsCount, aNodeInfo,
+                                           aLineNumber, aResult,
+                                           aAppendContent);
+#ifdef MOZ_XUL
   }
-  else
-    return nsXMLContentSink::CreateElement(aAtts, aAttsCount, aNameSpaceID, aNodeInfo, aResult);
+
+  *aAppendContent = PR_TRUE;
+  nsXULPrototypeElement* prototype = new nsXULPrototypeElement();
+  if (!prototype)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  prototype->mNodeInfo = aNodeInfo;
+
+  AddAttributesToXULPrototype(aAtts, aAttsCount, prototype);
+
+  nsresult rv = nsXULElement::Create(prototype, mDocument, PR_FALSE, aResult);
+
+  // XUL prototype elements start with a refcnt of 1 to represent
+  // ownership by the XUL prototype document.  In our case we have no
+  // prototype document, so release that reference.  The Create call
+  // above took a reference.
+  prototype->Release();
+
+  return rv;
+#endif
 }
 
 nsresult 
 nsXBLContentSink::AddAttributes(const PRUnichar** aAtts,
-                                nsIContent* aContent,
-                                PRBool aIsHTML)
+                                nsIContent* aContent)
 {
   if (aContent->IsContentOfType(nsIContent::eXUL))
     return NS_OK; // Nothing to do, since the proto already has the attrs.
-  else 
-    return nsXMLContentSink::AddAttributes(aAtts, aContent, aIsHTML);
+
+  return nsXMLContentSink::AddAttributes(aAtts, aContent);
 }
 
+#ifdef MOZ_XUL
 nsresult
 nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts, 
                                               PRUint32 aAttsCount, 
@@ -744,9 +934,10 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
 
   // Copy the attributes into the prototype
   nsCOMPtr<nsIAtom> nameSpacePrefix, nameAtom;
-  
-  for (; *aAtts; aAtts += 2) {
-    const nsDependentString key(aAtts[0]);
+
+  PRUint32 i;  
+  for (i = 0; i < aAttsCount; ++i) {
+    const nsDependentString key(aAtts[i * 2]);
 
     SplitXMLName(key, getter_AddRefs(nameSpacePrefix),
                  getter_AddRefs(nameAtom));
@@ -764,52 +955,25 @@ nsXBLContentSink::AddAttributesToXULPrototype(const PRUnichar **aAtts,
 
     if (kNameSpaceID_Unknown == nameSpaceID) {
       nameSpaceID = kNameSpaceID_None;
-      nameAtom = dont_AddRef(NS_NewAtom(key));
+      nameAtom = do_GetAtom(key);
       nameSpacePrefix = nsnull;
     } 
 
-    mNodeInfoManager->GetNodeInfo(nameAtom, nameSpacePrefix, nameSpaceID,
-                                  *getter_AddRefs(attrs->mNodeInfo));
+    if (nameSpaceID == kNameSpaceID_None) {
+      attrs[i].mName.SetTo(nameAtom);
+    }
+    else {
+      nsCOMPtr<nsINodeInfo> ni;
+      mNodeInfoManager->GetNodeInfo(nameAtom, nameSpacePrefix, nameSpaceID,
+                                    getter_AddRefs(ni));
+      attrs[i].mName.SetTo(ni);
+    }
     
-    attrs->mValue.SetValue(nsDependentString(aAtts[1]));
-    ++attrs;
-  }
-
-  // XUL elements may require some additional work to compute
-  // derived information.
-  if (aElement->mNodeInfo->NamespaceEquals(nsXULAtoms::nameSpaceID)) {
-    nsAutoString value;
-
-    // Compute the element's class list if the element has a 'class' attribute.
-    rv = aElement->GetAttr(kNameSpaceID_None, nsXULAtoms::clazz, value);
-    if (NS_FAILED(rv)) return rv;
-
-    if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
-      rv = nsClassList::ParseClasses(&aElement->mClassList, value);
-      if (NS_FAILED(rv)) return rv;
-    }
-
-    // Parse the element's 'style' attribute
-    rv = aElement->GetAttr(kNameSpaceID_None, nsHTMLAtoms::style, value);
-    if (NS_FAILED(rv)) return rv;
-
-    if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
-      if (!mCSSParser) {
-          rv = nsComponentManager::CreateInstance(kCSSParserCID,
-                                                  nsnull,
-                                                  NS_GET_IID(nsICSSParser),
-                                                  getter_AddRefs(mCSSParser));
-
-          if (NS_FAILED(rv)) return rv;
-      }
-
-      rv = mCSSParser->ParseStyleAttribute(value, mDocumentURL,
-                             getter_AddRefs(aElement->mInlineStyleRule));
-
-      NS_ASSERTION(NS_SUCCEEDED(rv), "unable to parse style rule");
-      if (NS_FAILED(rv)) return rv;
-    }
+    rv = aElement->SetAttrAt(i, nsDependentString(aAtts[i * 2 + 1]),
+                             mDocumentURI); 
+    NS_ENSURE_SUCCESS(rv, rv);
   }
 
   return NS_OK;
 }
+#endif
