@@ -1,7 +1,46 @@
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 sw=2 et tw=80: */
+/* ***** BEGIN LICENSE BLOCK *****
+ * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Netscape Public License
+ * Version 1.1 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/NPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ *
+ * The Original Code is Mozilla Communicator client code.
+ *
+ * The Initial Developer of the Original Code is 
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
+ *
+ * Contributor(s):
+ *
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the NPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the NPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
+
 #include "nsXBLDocumentInfo.h"
 #include "nsHashtable.h"
 #include "nsIDocument.h"
-#include "nsIXBLPrototypeBinding.h"
+#include "nsXBLPrototypeBinding.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIScriptContext.h"
 #include "nsIDOMScriptObjectFactory.h"
@@ -10,6 +49,10 @@
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
 #include "nsIChromeRegistry.h"
+#include "nsIPrincipal.h"
+#include "nsIPrincipalObsolete.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsContentUtils.h"
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID, NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
 
@@ -25,27 +68,32 @@ public:
   NS_DECL_ISUPPORTS
   
   // nsIScriptGlobalObject methods
-  NS_IMETHOD SetContext(nsIScriptContext *aContext);
-  NS_IMETHOD GetContext(nsIScriptContext **aContext);
-  NS_IMETHOD SetNewDocument(nsIDOMDocument *aDocument,
-                            PRBool removeEventListeners);
-  NS_IMETHOD SetDocShell(nsIDocShell *aDocShell);
-  NS_IMETHOD GetDocShell(nsIDocShell **aDocShell);
-  NS_IMETHOD SetOpenerWindow(nsIDOMWindowInternal *aOpener);
-  NS_IMETHOD SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner);
-  NS_IMETHOD GetGlobalObjectOwner(nsIScriptGlobalObjectOwner** aOwner);
-  NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext, 
-                            nsEvent* aEvent, 
-                            nsIDOMEvent** aDOMEvent,
-                            PRUint32 aFlags,
-                            nsEventStatus* aEventStatus);
-  NS_IMETHOD_(JSObject *) GetGlobalJSObject();
-  NS_IMETHOD OnFinalize(JSObject *aObject);
-  NS_IMETHOD SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
+  virtual void SetContext(nsIScriptContext *aContext);
+  virtual nsIScriptContext *GetContext();
+  virtual nsresult SetNewDocument(nsIDOMDocument *aDocument,
+                                  PRBool aRemoveEventListeners,
+                                  PRBool aClearScope);
+  virtual void SetDocShell(nsIDocShell *aDocShell);
+  virtual nsIDocShell *GetDocShell();
+  virtual void SetOpenerWindow(nsIDOMWindowInternal *aOpener);
+  virtual void SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner);
+  virtual nsIScriptGlobalObjectOwner *GetGlobalObjectOwner();
+  virtual nsresult HandleDOMEvent(nsIPresContext* aPresContext, 
+                                  nsEvent* aEvent, 
+                                  nsIDOMEvent** aDOMEvent,
+                                  PRUint32 aFlags,
+                                  nsEventStatus* aEventStatus);
+  virtual JSObject *GetGlobalJSObject();
+  virtual void OnFinalize(JSObject *aObject);
+  virtual void SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
 
   // nsIScriptObjectPrincipal methods
+  NS_IMETHOD GetPrincipalObsolete(nsIPrincipalObsolete** aPrincipal);
   NS_IMETHOD GetPrincipal(nsIPrincipal** aPrincipal);
     
+  static JSBool doCheckAccess(JSContext *cx, JSObject *obj, jsval id,
+                              PRUint32 accessType);
+
 protected:
   virtual ~nsXBLDocGlobalObject();
 
@@ -56,7 +104,63 @@ protected:
   static JSClass gSharedGlobalClass;
 };
 
-void PR_CALLBACK nsXBLDocGlobalObject_finalize(JSContext *cx, JSObject *obj)
+JSBool
+nsXBLDocGlobalObject::doCheckAccess(JSContext *cx, JSObject *obj, jsval id, PRUint32 accessType)
+{
+  nsIScriptSecurityManager *ssm = nsContentUtils::GetSecurityManager();
+  if (!ssm) {
+    ::JS_ReportError(cx, "Unable to verify access to a global object property.");
+    return JS_FALSE;
+  }
+
+  // Make sure to actually operate on our object, and not some object further
+  // down on the proto chain.
+  while (JS_GET_CLASS(cx, obj) != &nsXBLDocGlobalObject::gSharedGlobalClass) {
+    obj = ::JS_GetPrototype(cx, obj);
+    if (!obj) {
+      ::JS_ReportError(cx, "Invalid access to a global object property.");
+      return JS_FALSE;
+    }
+  }
+
+  nsresult rv = ssm->CheckPropertyAccess(cx, obj, JS_GET_CLASS(cx, obj)->name,
+                                         id, accessType);
+  return NS_SUCCEEDED(rv);
+}
+
+PR_STATIC_CALLBACK(JSBool)
+nsXBLDocGlobalObject_getProperty(JSContext *cx, JSObject *obj,
+                                 jsval id, jsval *vp)
+{
+  return nsXBLDocGlobalObject::
+    doCheckAccess(cx, obj, id, nsIXPCSecurityManager::ACCESS_GET_PROPERTY);
+}
+
+PR_STATIC_CALLBACK(JSBool)
+nsXBLDocGlobalObject_setProperty(JSContext *cx, JSObject *obj,
+                                 jsval id, jsval *vp)
+{
+  return nsXBLDocGlobalObject::
+    doCheckAccess(cx, obj, id, nsIXPCSecurityManager::ACCESS_SET_PROPERTY);
+}
+
+PR_STATIC_CALLBACK(JSBool)
+nsXBLDocGlobalObject_checkAccess(JSContext *cx, JSObject *obj, jsval id,
+                                 JSAccessMode mode, jsval *vp)
+{
+  PRUint32 translated;
+  if (mode & JSACC_WRITE) {
+    translated = nsIXPCSecurityManager::ACCESS_SET_PROPERTY;
+  } else {
+    translated = nsIXPCSecurityManager::ACCESS_GET_PROPERTY;
+  }
+
+  return nsXBLDocGlobalObject::
+    doCheckAccess(cx, obj, id, translated);
+}
+
+PR_STATIC_CALLBACK(void)
+nsXBLDocGlobalObject_finalize(JSContext *cx, JSObject *obj)
 {
   nsISupports *nativeThis = (nsISupports*)JS_GetPrivate(cx, obj);
 
@@ -69,8 +173,8 @@ void PR_CALLBACK nsXBLDocGlobalObject_finalize(JSContext *cx, JSObject *obj)
   NS_RELEASE(nativeThis);
 }
 
-
-JSBool PR_CALLBACK nsXBLDocGlobalObject_resolve(JSContext *cx, JSObject *obj, jsval id)
+PR_STATIC_CALLBACK(JSBool)
+nsXBLDocGlobalObject_resolve(JSContext *cx, JSObject *obj, jsval id)
 {
   JSBool did_resolve = JS_FALSE;
   return JS_ResolveStandardClass(cx, obj, id, &did_resolve);
@@ -80,9 +184,11 @@ JSBool PR_CALLBACK nsXBLDocGlobalObject_resolve(JSContext *cx, JSObject *obj, js
 JSClass nsXBLDocGlobalObject::gSharedGlobalClass = {
     "nsXBLPrototypeScript compilation scope",
     JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS,
-    JS_PropertyStub,  JS_PropertyStub, JS_PropertyStub, JS_PropertyStub,
-    JS_EnumerateStub, nsXBLDocGlobalObject_resolve,  JS_ConvertStub,
-    nsXBLDocGlobalObject_finalize
+    JS_PropertyStub,  JS_PropertyStub,
+    nsXBLDocGlobalObject_getProperty, nsXBLDocGlobalObject_setProperty,
+    JS_EnumerateStub, nsXBLDocGlobalObject_resolve,
+    JS_ConvertStub, nsXBLDocGlobalObject_finalize,
+    NULL, nsXBLDocGlobalObject_checkAccess
 };
 
 //----------------------------------------------------------------------
@@ -94,7 +200,6 @@ nsXBLDocGlobalObject::nsXBLDocGlobalObject()
     : mJSObject(nsnull),
       mGlobalObjectOwner(nsnull)
 {
-  NS_INIT_REFCNT();
 }
 
 
@@ -135,7 +240,7 @@ XBL_ProtoErrorReporter(JSContext *cx,
 // nsIScriptGlobalObject methods
 //
 
-NS_IMETHODIMP
+void
 nsXBLDocGlobalObject::SetContext(nsIScriptContext *aContext)
 {
   mScriptContext = aContext;
@@ -143,29 +248,28 @@ nsXBLDocGlobalObject::SetContext(nsIScriptContext *aContext)
     JSContext* cx = (JSContext *)mScriptContext->GetNativeContext();
     JS_SetErrorReporter(cx, XBL_ProtoErrorReporter);
   }
-  return NS_OK;
 }
 
 
-NS_IMETHODIMP
-nsXBLDocGlobalObject::GetContext(nsIScriptContext **aContext)
+nsIScriptContext *
+nsXBLDocGlobalObject::GetContext()
 {
   // This whole fragile mess is predicated on the fact that
   // GetContext() will be called before GetScriptObject() is.
   if (! mScriptContext) {
     nsCOMPtr<nsIDOMScriptObjectFactory> factory = do_GetService(kDOMScriptObjectFactoryCID);
-    NS_ENSURE_TRUE(factory, NS_ERROR_FAILURE);
+    NS_ENSURE_TRUE(factory, nsnull);
 
     nsresult rv =  factory->NewScriptContext(nsnull, getter_AddRefs(mScriptContext));
     if (NS_FAILED(rv))
-        return rv;
+        return nsnull;
 
     JSContext *cx = (JSContext *)mScriptContext->GetNativeContext();
 
     JS_SetErrorReporter(cx, XBL_ProtoErrorReporter);
     mJSObject = ::JS_NewObject(cx, &gSharedGlobalClass, nsnull, nsnull);
     if (!mJSObject)
-        return NS_ERROR_OUT_OF_MEMORY;
+        return nsnull;
 
     ::JS_SetGlobalObject(cx, mJSObject);
 
@@ -175,63 +279,57 @@ nsXBLDocGlobalObject::GetContext(nsIScriptContext **aContext)
     NS_ADDREF(this);
   }
 
-  *aContext = mScriptContext;
-  NS_IF_ADDREF(*aContext);
-  return NS_OK;
+  return mScriptContext;
 }
 
 
-NS_IMETHODIMP
+nsresult
 nsXBLDocGlobalObject::SetNewDocument(nsIDOMDocument *aDocument,
-                                     PRBool removeEventListeners)
+                                     PRBool aRemoveEventListeners,
+                                     PRBool aClearScope)
 {
   NS_NOTREACHED("waaah!");
   return NS_ERROR_UNEXPECTED;
 }
 
 
-NS_IMETHODIMP
+void
 nsXBLDocGlobalObject::SetDocShell(nsIDocShell *aDocShell)
 {
   NS_NOTREACHED("waaah!");
-  return NS_ERROR_UNEXPECTED;
 }
 
 
-NS_IMETHODIMP
-nsXBLDocGlobalObject::GetDocShell(nsIDocShell **aDocShell)
+nsIDocShell *
+nsXBLDocGlobalObject::GetDocShell()
 {
   NS_WARNING("waaah!");
-  return NS_ERROR_UNEXPECTED;
+  return nsnull;
 }
 
 
-NS_IMETHODIMP
+void
 nsXBLDocGlobalObject::SetOpenerWindow(nsIDOMWindowInternal *aOpener)
 {
   NS_NOTREACHED("waaah!");
-  return NS_ERROR_UNEXPECTED;
 }
 
 
-NS_IMETHODIMP
+void
 nsXBLDocGlobalObject::SetGlobalObjectOwner(nsIScriptGlobalObjectOwner* aOwner)
 {
   mGlobalObjectOwner = aOwner; // weak reference
-  return NS_OK;
 }
 
 
-NS_IMETHODIMP
-nsXBLDocGlobalObject::GetGlobalObjectOwner(nsIScriptGlobalObjectOwner** aOwner)
+nsIScriptGlobalObjectOwner *
+nsXBLDocGlobalObject::GetGlobalObjectOwner()
 {
-  *aOwner = mGlobalObjectOwner;
-  NS_IF_ADDREF(*aOwner);
-  return NS_OK;
+  return mGlobalObjectOwner;
 }
 
 
-NS_IMETHODIMP
+nsresult
 nsXBLDocGlobalObject::HandleDOMEvent(nsIPresContext* aPresContext, 
                                        nsEvent* aEvent, 
                                        nsIDOMEvent** aDOMEvent,
@@ -242,7 +340,7 @@ nsXBLDocGlobalObject::HandleDOMEvent(nsIPresContext* aPresContext,
   return NS_ERROR_UNEXPECTED;
 }
 
-NS_IMETHODIMP_(JSObject *)
+JSObject *
 nsXBLDocGlobalObject::GetGlobalJSObject()
 {
   // The prototype document has its own special secret script object
@@ -259,22 +357,18 @@ nsXBLDocGlobalObject::GetGlobalJSObject()
   return ::JS_GetGlobalObject(cx);
 }
 
-NS_IMETHODIMP
+void
 nsXBLDocGlobalObject::OnFinalize(JSObject *aObject)
 {
   NS_ASSERTION(aObject == mJSObject, "Wrong object finalized!");
 
   mJSObject = nsnull;
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsXBLDocGlobalObject::SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts)
 {
     // We don't care...
-
-    return NS_OK;
 }
 
 
@@ -283,6 +377,19 @@ nsXBLDocGlobalObject::SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts)
 // nsIScriptObjectPrincipal methods
 //
 
+NS_IMETHODIMP
+nsXBLDocGlobalObject::GetPrincipalObsolete(nsIPrincipalObsolete** aPrincipal)
+{
+  nsCOMPtr<nsIPrincipal> principal;
+  nsresult rv = nsXBLDocGlobalObject::GetPrincipal(getter_AddRefs(principal));
+  if (principal)
+    CallQueryInterface(principal, aPrincipal);
+  else
+    *aPrincipal = nsnull;
+
+  return rv;
+}
+
 NS_IMETHODIMP
 nsXBLDocGlobalObject::GetPrincipal(nsIPrincipal** aPrincipal)
 {
@@ -299,7 +406,12 @@ nsXBLDocGlobalObject::GetPrincipal(nsIPrincipal** aPrincipal)
   rv = docInfo->GetDocument(getter_AddRefs(document));
   NS_ENSURE_SUCCESS(rv, NS_ERROR_FAILURE);
 
-  return document->GetPrincipal(aPrincipal);
+  *aPrincipal = document->GetPrincipal();
+  if (!*aPrincipal)
+    return NS_ERROR_FAILURE;
+
+  NS_ADDREF(*aPrincipal);
+  return NS_OK;
 }
 
 static PRBool IsChromeOrResourceURI(nsIURI* aURI)
@@ -315,22 +427,15 @@ static PRBool IsChromeOrResourceURI(nsIURI* aURI)
 /* Implementation file */
 NS_IMPL_ISUPPORTS3(nsXBLDocumentInfo, nsIXBLDocumentInfo, nsIScriptGlobalObjectOwner, nsISupportsWeakReference)
 
-static NS_DEFINE_CID(kChromeRegistryCID,          NS_CHROMEREGISTRY_CID);
-
-nsXBLDocumentInfo::nsXBLDocumentInfo(const char* aDocURI, nsIDocument* aDocument)
+nsXBLDocumentInfo::nsXBLDocumentInfo(nsIDocument* aDocument)
+  : mDocument(aDocument),
+    mScriptAccess(PR_TRUE),
+    mBindingTable(nsnull)
 {
-  NS_INIT_ISUPPORTS();
-  /* member initializers and constructor code */
-  mDocURI = aDocURI;
-  mDocument = aDocument;
-  mScriptAccess = PR_TRUE;
-  mBindingTable = nsnull;
-
-  nsCOMPtr<nsIURI> uri;
-  mDocument->GetDocumentURL(getter_AddRefs(uri));
+  nsIURI* uri = aDocument->GetDocumentURI();
   if (IsChromeOrResourceURI(uri)) {
     // Cache whether or not this chrome XBL can execute scripts.
-    nsCOMPtr<nsIChromeRegistry> reg(do_GetService(kChromeRegistryCID));
+    nsCOMPtr<nsIXULChromeRegistry> reg(do_GetService(NS_CHROMEREGISTRY_CONTRACTID));
     if (reg) {
       PRBool allow = PR_TRUE;
       reg->AllowScriptsForSkin(uri, &allow);
@@ -350,7 +455,7 @@ nsXBLDocumentInfo::~nsXBLDocumentInfo()
 }
 
 NS_IMETHODIMP
-nsXBLDocumentInfo::GetPrototypeBinding(const nsACString& aRef, nsIXBLPrototypeBinding** aResult)
+nsXBLDocumentInfo::GetPrototypeBinding(const nsACString& aRef, nsXBLPrototypeBinding** aResult)
 {
   *aResult = nsnull;
   if (!mBindingTable)
@@ -358,16 +463,24 @@ nsXBLDocumentInfo::GetPrototypeBinding(const nsACString& aRef, nsIXBLPrototypeBi
 
   const nsPromiseFlatCString& flat = PromiseFlatCString(aRef);
   nsCStringKey key(flat.get());
-  *aResult = NS_STATIC_CAST(nsIXBLPrototypeBinding*, mBindingTable->Get(&key)); // Addref happens here.
+  *aResult = NS_STATIC_CAST(nsXBLPrototypeBinding*, mBindingTable->Get(&key));
 
   return NS_OK;
 }
 
+static PRBool PR_CALLBACK
+DeletePrototypeBinding(nsHashKey* aKey, void* aData, void* aClosure)
+{
+  nsXBLPrototypeBinding* binding = NS_STATIC_CAST(nsXBLPrototypeBinding*, aData);
+  delete binding;
+  return PR_TRUE;
+}
+
 NS_IMETHODIMP
-nsXBLDocumentInfo::SetPrototypeBinding(const nsACString& aRef, nsIXBLPrototypeBinding* aBinding)
+nsXBLDocumentInfo::SetPrototypeBinding(const nsACString& aRef, nsXBLPrototypeBinding* aBinding)
 {
   if (!mBindingTable)
-    mBindingTable = new nsSupportsHashtable();
+    mBindingTable = new nsObjectHashtable(nsnull, nsnull, DeletePrototypeBinding, nsnull);
 
   const nsPromiseFlatCString& flat = PromiseFlatCString(aRef);
   nsCStringKey key(flat.get());
@@ -378,7 +491,7 @@ nsXBLDocumentInfo::SetPrototypeBinding(const nsACString& aRef, nsIXBLPrototypeBi
 
 PRBool PR_CALLBACK FlushScopedSkinSheets(nsHashKey* aKey, void* aData, void* aClosure)
 {
-  nsIXBLPrototypeBinding* proto = (nsIXBLPrototypeBinding*)aData;
+  nsXBLPrototypeBinding* proto = (nsXBLPrototypeBinding*)aData;
   proto->FlushSkinSheets();
   return PR_TRUE;
 }
@@ -433,15 +546,14 @@ nsXBLDocumentInfo::ReportScriptError(nsIScriptError *errorObject)
 
 nsresult NS_NewXBLDocumentInfo(nsIDocument* aDocument, nsIXBLDocumentInfo** aResult)
 {
-  nsCOMPtr<nsIURI> url;
-  aDocument->GetDocumentURL(getter_AddRefs(url));
-  
-  nsCAutoString str;
-  url->GetSpec(str);
+  NS_PRECONDITION(aDocument, "Must have a document!");
 
-  *aResult = new nsXBLDocumentInfo(str.get(), aDocument);
-  
-  NS_IF_ADDREF(*aResult);
+  *aResult = new nsXBLDocumentInfo(aDocument);
+  if (!*aResult) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  NS_ADDREF(*aResult);
   return NS_OK;
 }
 
