@@ -61,13 +61,13 @@
 #define __LITTLE_ENDIAN
 #endif
 
-#if defined(linux) && defined(__i386__)
+#if defined(linux) && (defined(__i386__) || defined(__x86_64__))
 #define __LITTLE_ENDIAN
 #endif
 
 /* End here. The rest is the standard file. */
 
-#ifdef __NEWVALID	/* special setup for Sun test regime */
+#ifdef SOLARIS /* special setup for Sun test regime */
 #if defined(i386) || defined(i486) || \
 	defined(intel) || defined(x86) || defined(i86pc)
 #define __LITTLE_ENDIAN
