@@ -25,6 +25,11 @@
   Text import addressbook interfaces
 
 */
+#ifdef MOZ_LOGGING
+// sorry, this has to be before the pre-compiled header
+#define FORCE_PR_LOG /* Allow logging in the release build */
+#endif
+
 #include "nscore.h"
 #include "nsCRT.h"
 #include "nsString.h"
@@ -50,8 +55,7 @@
 #include "TextDebugLog.h"
 
 static NS_DEFINE_IID(kISupportsIID,			NS_ISUPPORTS_IID);
-static NS_DEFINE_CID(kPrefServiceCID,		NS_PREF_CID);
-
+PRLogModuleInfo* TEXTIMPORTLOGMODULE;
 
 class ImportAddressImpl : public nsIImportAddressBooks
 {
@@ -124,8 +128,9 @@ private:
 
 nsTextImport::nsTextImport()
 {
-    NS_INIT_REFCNT();
-
+  // Init logging module.
+  if (!TEXTIMPORTLOGMODULE)
+    TEXTIMPORTLOGMODULE = PR_NewLogModule("IMPORT");
 	IMPORT_LOG0( "nsTextImport Module Created\n");
 
 	nsTextStringBundle::GetStringBundle();
@@ -243,8 +248,6 @@ nsresult ImportAddressImpl::Create(nsIImportAddressBooks** aImport)
 
 ImportAddressImpl::ImportAddressImpl()
 {
-    NS_INIT_REFCNT();
-
 	m_fileLoc = nsnull;
 	m_haveDelim = PR_FALSE;
 }
@@ -408,7 +411,7 @@ void ImportAddressImpl::ReportSuccess( nsString& name, nsString *pStream)
 	pStream->Append( pText);
 	nsTextFormatter::smprintf_free( pText);
 	nsTextStringBundle::FreeString( pFmt);
-	pStream->AppendWithConversion( char(nsCRT::LF));
+	pStream->Append( PRUnichar(nsCRT::LF));
 	NS_IF_RELEASE( pBundle);
 }
 
@@ -423,7 +426,7 @@ void ImportAddressImpl::ReportError( PRInt32 errorNum, nsString& name, nsString
 	pStream->Append( pText);
 	nsTextFormatter::smprintf_free( pText);
 	nsTextStringBundle::FreeString( pFmt);
-	pStream->AppendWithConversion( char(nsCRT::LF));
+	pStream->Append( PRUnichar(nsCRT::LF));
 	NS_IF_RELEASE( pBundle);
 }
 
@@ -489,12 +492,9 @@ NS_IMETHODIMP ImportAddressImpl::ImportAddressBook(	nsIImportABDescriptor *pSour
     	return( NS_ERROR_FAILURE);
     }
 
-#ifdef IMPORT_DEBUG
-	char *pPath;
-	inFile->GetNativePath( &pPath);    
-	IMPORT_LOG1( "Import address book: %s\n", pPath);
-	nsCRT::free( pPath);
-#endif
+  nsXPIDLCString pPath; 
+  inFile->GetNativePath(getter_Copies(pPath));
+	IMPORT_LOG1( "Importing address book: %s\n", pPath.get());
 	
     nsresult rv = NS_OK;
 	PRBool	isLDIF = PR_FALSE;
@@ -695,7 +695,7 @@ NS_IMETHODIMP ImportAddressImpl::InitFieldMap(nsIFileSpec *location, nsIImportFi
 	// from the same file format.
 	
 	nsresult rv;
-	nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
+	nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
 	if (NS_SUCCEEDED( rv)) {
 		nsXPIDLCString	prefStr;
 		rv = prefs->CopyCharPref( "mailnews.import.text.fieldmap", getter_Copies(prefStr));
@@ -774,7 +774,7 @@ void ImportAddressImpl::SaveFieldMap( nsIImportFieldMap *pMap)
 	PRBool	done = PR_FALSE;
 	nsresult rv;
 	// NS_WITH_PROXIED_SERVICE( nsIPref, prefs, kPrefServiceCID, NS_UI_THREAD_EVENTQ, &rv);
-	nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
+  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
 
 	if (NS_SUCCEEDED( rv)) {
 		nsXPIDLCString	prefStr;
