@@ -129,8 +129,8 @@ static PLDHashTableOps gMapOps = {
 
 
 nsSecureBrowserUIImpl::nsSecureBrowserUIImpl()
-  : mIsViewSource(PR_FALSE),
-    mPreviousSecurityState(lis_no_security)
+  : mPreviousSecurityState(lis_no_security),
+    mIsViewSource(PR_FALSE)
 {
   mTransferringRequests.ops = nsnull;
   mNewToplevelSecurityState = STATE_IS_INSECURE;
@@ -374,6 +374,79 @@ void nsSecureBrowserUIImpl::ResetStateTracking()
                     sizeof(RequestHashEntry), 16);
 }
 
+nsresult
+nsSecureBrowserUIImpl::EvaluateAndUpdateSecurityState(nsIRequest *aRequest)
+{
+  nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
+
+  if (!channel) {
+    mNewToplevelSecurityState = nsIWebProgressListener::STATE_IS_INSECURE;
+  } else {
+    mNewToplevelSecurityState = GetSecurityStateFromChannel(channel);
+
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: OnStateChange: remember mNewToplevelSecurityState => %x\n", this,
+            mNewToplevelSecurityState));
+
+    // Get SSL Status information if possible
+    nsCOMPtr<nsISupports> info;
+    channel->GetSecurityInfo(getter_AddRefs(info));
+    nsCOMPtr<nsISSLStatusProvider> sp = do_QueryInterface(info);
+    if (sp) {
+      // Ignore result
+      sp->GetSSLStatus(getter_AddRefs(mSSLStatus));
+    }
+
+    if (info) {
+      nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
+      if (secInfo) {
+        secInfo->GetShortSecurityDescription(getter_Copies(mInfoTooltip));
+      }
+    }
+  }
+
+  // assume mNewToplevelSecurityState was set in this scope!
+  // see code that is directly above
+
+  mNewToplevelSecurityStateKnown = PR_TRUE;
+  return UpdateSecurityState(aRequest);
+}
+
+void
+nsSecureBrowserUIImpl::UpdateSubrequestMembers(nsIRequest *aRequest)
+{
+  // For wyciwyg channels in subdocuments we only update our
+  // subrequest state members.
+  PRUint32 reqState = nsIWebProgressListener::STATE_IS_INSECURE;
+  nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
+
+  if (channel) {
+    reqState = GetSecurityStateFromChannel(channel);
+  }
+
+  if (reqState & STATE_IS_SECURE) {
+    if (reqState & STATE_SECURE_LOW || reqState & STATE_SECURE_MED) {
+      PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+             ("SecureUI:%p: OnStateChange: subreq LOW\n", this));
+      ++mSubRequestsLowSecurity;
+    } else {
+      PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+             ("SecureUI:%p: OnStateChange: subreq HIGH\n", this));
+      ++mSubRequestsHighSecurity;
+    }
+  } else if (reqState & STATE_IS_BROKEN) {
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: OnStateChange: subreq BROKEN\n", this));
+    ++mSubRequestsBrokenSecurity;
+  } else {
+    PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
+           ("SecureUI:%p: OnStateChange: subreq INSECURE\n", this));
+    ++mSubRequestsNoSecurity;
+  }
+}
+
+
+
 NS_IMETHODIMP
 nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
                                      nsIRequest* aRequest,
@@ -787,50 +860,11 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
 
     --mDocumentRequestsInProgress;
 
-    {
-      PRBool MustEvaluate = PR_TRUE;
+    if (requestHasTransferedData) {
+      // Data has been transferred for the single toplevel
+      // request. Evaluate the security state.
 
-      if (!requestHasTransferedData)
-      {
-        // No data has been transfered for the single toplevel request.
-        MustEvaluate = PR_FALSE;
-      }
-
-      if (MustEvaluate)
-      {
-        if (channel) {
-          mNewToplevelSecurityState = GetSecurityStateFromChannel(channel);
-
-          PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-                 ("SecureUI:%p: OnStateChange: remember mNewToplevelSecurityState => %x\n", this,
-                  mNewToplevelSecurityState));
-
-          // Get SSL Status information if possible
-          nsCOMPtr<nsISupports> info;
-          channel->GetSecurityInfo(getter_AddRefs(info));
-          nsCOMPtr<nsISSLStatusProvider> sp = do_QueryInterface(info);
-          if (sp) {
-            // Ignore result
-            sp->GetSSLStatus(getter_AddRefs(mSSLStatus));
-          }
-
-          if (info) {
-            nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
-            if (secInfo) {
-                secInfo->GetShortSecurityDescription(getter_Copies(mInfoTooltip));
-            }
-          }
-        }
-        else {
-          mNewToplevelSecurityState = nsIWebProgressListener::STATE_IS_INSECURE;
-        }
-
-        // assume mNewToplevelSecurityState was set in this scope!
-        // see code that is directly above
-        
-        mNewToplevelSecurityStateKnown = PR_TRUE;
-        return UpdateSecurityState(aRequest);
-      }
+      return EvaluateAndUpdateSecurityState(aRequest);
     }
     
     return NS_OK;
@@ -849,39 +883,7 @@ nsSecureBrowserUIImpl::OnStateChange(nsIWebProgress* aWebProgress,
 
     if (requestHasTransferedData)
     {  
-      PRUint32 reqState = nsIWebProgressListener::STATE_IS_INSECURE;
-
-      if (channel) {
-        reqState = GetSecurityStateFromChannel(channel);
-      }
-
-      if (reqState & STATE_IS_SECURE)
-      {
-        if (reqState & STATE_SECURE_LOW || reqState & STATE_SECURE_MED)
-        {
-          PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-           ("SecureUI:%p: OnStateChange: subreq LOW\n", this));
-          ++mSubRequestsLowSecurity;
-        }
-        else
-        {
-          PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-           ("SecureUI:%p: OnStateChange: subreq HIGH\n", this));
-          ++mSubRequestsHighSecurity;
-        }
-      }
-      else if (reqState & STATE_IS_BROKEN)
-      {
-        PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-         ("SecureUI:%p: OnStateChange: subreq BROKEN\n", this));
-        ++ mSubRequestsBrokenSecurity;
-      }
-      else
-      {
-        PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
-         ("SecureUI:%p: OnStateChange: subreq INSECURE\n", this));
-        ++mSubRequestsNoSecurity;
-      }
+      UpdateSubrequestMembers(aRequest);
       
       // Care for the following scenario:
       // A new top level document load might have already started,
@@ -919,6 +921,9 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
 {
   lockIconState newSecurityState;
 
+  PRBool showWarning = PR_FALSE;
+  lockIconState warnSecurityState;
+
   if (mNewToplevelSecurityState & STATE_IS_SECURE)
   {
     if (mNewToplevelSecurityState & STATE_SECURE_LOW
@@ -1012,7 +1017,7 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
       high        high
     */
 
-    PRBool showWarning = PR_TRUE;
+    showWarning = PR_TRUE;
     
     switch (mPreviousSecurityState)
     {
@@ -1032,30 +1037,12 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
       default:
         break;
     }
-    
+
     if (showWarning)
     {
-      switch (newSecurityState)
-      {
-        case lis_no_security:
-        case lis_broken_security:
-          ConfirmLeavingSecure();
-          break;
-
-        case lis_mixed_security:
-          ConfirmMixedMode();
-          break;
-
-        case lis_low_security:
-          ConfirmEnteringWeak();
-          break;
-
-        case lis_high_security:
-          ConfirmEnteringSecure();
-          break;
-      }
+      warnSecurityState = newSecurityState;
     }
-
+    
     mPreviousSecurityState = newSecurityState;
 
     if (lis_no_security == newSecurityState)
@@ -1108,6 +1095,29 @@ nsresult nsSecureBrowserUIImpl::UpdateSecurityState(nsIRequest* aRequest)
 
   }
 
+  if (showWarning)
+  {
+    switch (warnSecurityState)
+    {
+      case lis_no_security:
+      case lis_broken_security:
+        ConfirmLeavingSecure();
+        break;
+
+      case lis_mixed_security:
+        ConfirmMixedMode();
+        break;
+
+      case lis_low_security:
+        ConfirmEnteringWeak();
+        break;
+
+      case lis_high_security:
+        ConfirmEnteringSecure();
+        break;
+    }
+  }
+
   return NS_OK; 
 }
 
@@ -1116,21 +1126,61 @@ nsSecureBrowserUIImpl::OnLocationChange(nsIWebProgress* aWebProgress,
                                         nsIRequest* aRequest,
                                         nsIURI* aLocation)
 {
-  mCurrentURI = aLocation;
-
-  if (mCurrentURI)
+  if (aLocation)
   {
     PRBool vs;
-    
-    if (NS_SUCCEEDED(mCurrentURI->SchemeIs("view-source", &vs)) && vs)
-    {
+
+    nsresult rv = aLocation->SchemeIs("view-source", &vs);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    if (vs) {
       PR_LOG(gSecureDocLog, PR_LOG_DEBUG,
              ("SecureUI:%p: OnLocationChange: view-source\n", this));
-
-      mIsViewSource = PR_TRUE;
     }
+
+    mIsViewSource = vs;
   }
 
+  mCurrentURI = aLocation;
+
+  // If the location change does not have a corresponding request, then we
+  // assume that it does not impact the security state.
+  if (!aRequest)
+    return NS_OK;
+
+  // The location bar has changed, so we must update the security state.  The
+  // only concern with doing this here is that a page may transition from being
+  // reported as completely secure to being reported as partially secure
+  // (mixed).  This may be confusing for users, and it may bother users who
+  // like seeing security dialogs.  However, it seems prudent given that page
+  // loading may never end in some edge cases (perhaps by a site with malicious
+  // intent).
+
+  nsCOMPtr<nsIDOMWindow> windowForProgress;
+  aWebProgress->GetDOMWindow(getter_AddRefs(windowForProgress));
+
+  if (windowForProgress.get() == mWindow.get()) {
+    // For toplevel channels, update the security state right away.
+    return EvaluateAndUpdateSecurityState(aRequest);
+  }
+
+  // For channels in subdocuments we only update our subrequest state members.
+  UpdateSubrequestMembers(aRequest);
+
+  // Care for the following scenario:
+
+  // A new toplevel document load might have already started, but the security
+  // state of the new toplevel document might not yet be known.
+  // 
+  // At this point, we are learning about the security state of a sub-document.
+  // We must not update the security state based on the sub content, if the new
+  // top level state is not yet known.
+  //
+  // We skip updating the security state in this case.
+
+  if (mNewToplevelSecurityStateKnown)
+    return UpdateSecurityState(aRequest);
+
   return NS_OK;
 }
 
