@@ -38,10 +38,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-#include "nsPhWidgetLog.h"
 #include <Pt.h>
-#include <PtServer.h>
+#include <photon/PtServer.h>
 #include "PtRawDrawContainer.h"
+#include "nsCRT.h"
 
 #include "nsWindow.h"
 #include "nsWidgetsCID.h"
@@ -71,9 +71,21 @@
 #include "nsIPresShell.h"
 #include "nsReadableUtils.h"
 
-PRBool             nsWindow::mResizeQueueInited = PR_FALSE;
-DamageQueueEntry  *nsWindow::mResizeQueue = nsnull;
-PtWorkProcId_t    *nsWindow::mResizeProcID = nsnull;
+static PhTile_t *GetWindowClipping( PtWidget_t *aWidget );
+
+nsIRollupListener *nsWindow::gRollupListener = nsnull;
+nsIWidget         *nsWindow::gRollupWidget = nsnull;
+static PtWidget_t	*gMenuRegion;
+
+/* avoid making a costly PhWindowQueryVisible call */
+static PhRect_t gConsoleRect;
+static PRBool gConsoleRectValid = PR_FALSE;
+#define QueryVisible( )	{\
+													if( gConsoleRectValid == PR_FALSE ) { \
+															PhWindowQueryVisible( Ph_QUERY_GRAPHICS, 0, 1, &gConsoleRect ); \
+															gConsoleRectValid = PR_TRUE;\
+															} \
+													}
 
 //-------------------------------------------------------------------------
 //
@@ -84,25 +96,9 @@ nsWindow::nsWindow()
 {
   mClientWidget    = nsnull;
   mIsTooSmall      = PR_FALSE;
-  mBlockFocusEvents = PR_FALSE;
   mBorderStyle     = eBorderStyle_default;
   mWindowType      = eWindowType_child;
-  mIsResizing      = PR_FALSE;
-}
-
-ChildWindow::ChildWindow()
-{
-  mBorderStyle     = eBorderStyle_none;
-  mWindowType      = eWindowType_child;
-}
-
-ChildWindow::~ChildWindow()
-{
-}
-
-PRBool ChildWindow::IsChild() const
-{
-  return PR_TRUE;
+	mLastMenu				 = nsnull;
 }
 
 //-------------------------------------------------------------------------
@@ -112,11 +108,12 @@ PRBool ChildWindow::IsChild() const
 //-------------------------------------------------------------------------
 nsWindow::~nsWindow()
 {
+	nsWindow *p = (nsWindow*)mParent;
+	if( p && p->mLastMenu == mWidget ) p->mLastMenu = nsnull;
+
   // always call destroy.  if it's already been called, there's no harm
   // since it keeps track of when it's already been called.
   Destroy();
-
-  RemoveResizeWidget();
 }
 
 PRBool nsWindow::IsChild() const
@@ -126,13 +123,15 @@ PRBool nsWindow::IsChild() const
 
 NS_IMETHODIMP nsWindow::WidgetToScreen( const nsRect& aOldRect, nsRect& aNewRect ) {
 
-	PhPoint_t pos, offset, orig = nsToolkit::GetConsoleOffset();
+	PhPoint_t pos, offset;
 	PtWidget_t *disjoint = PtFindDisjoint( mWidget );
 
+	QueryVisible( );
+
 	PtGetAbsPosition( disjoint, &pos.x, &pos.y );
 	PtWidgetOffset( mWidget, &offset );
-	aNewRect.x = pos.x + offset.x + aOldRect.x - orig.x;
-	aNewRect.y = pos.y + offset.y + aOldRect.y - orig.y;
+	aNewRect.x = pos.x + offset.x + aOldRect.x - gConsoleRect.ul.x;
+	aNewRect.y = pos.y + offset.y + aOldRect.y - gConsoleRect.ul.y;
 
 	aNewRect.width = aOldRect.width;
 	aNewRect.height = aOldRect.height;
@@ -142,7 +141,6 @@ NS_IMETHODIMP nsWindow::WidgetToScreen( const nsRect& aOldRect, nsRect& aNewRect
 
 void nsWindow::DestroyNative(void)
 {
-  RemoveResizeWidget();
   if ( mClientWidget )
   {
     PtDestroyWidget(mClientWidget );  
@@ -150,7 +148,7 @@ void nsWindow::DestroyNative(void)
   }
   // destroy all of the children that are nsWindow() classes
   // preempting the gdk destroy system.
-  DestroyNativeChildren();
+  if( mWidget ) DestroyNativeChildren();
 
   // Call the base class to actually PtDestroy mWidget.
   nsWidget::DestroyNative();
@@ -172,11 +170,6 @@ void nsWindow::DestroyNativeChildren(void)
   }
 }
 
-NS_IMETHODIMP nsWindow::Update(void)
-{
-  return nsWidget::Update();
-}
-
 NS_IMETHODIMP nsWindow::CaptureRollupEvents( nsIRollupListener * aListener, PRBool aDoCapture, PRBool aConsumeRollupEvent ) {
   PtWidget_t *grabWidget;
   grabWidget = mWidget;
@@ -188,59 +181,45 @@ NS_IMETHODIMP nsWindow::CaptureRollupEvents( nsIRollupListener * aListener, PRBo
     NS_ADDREF(aListener);
     gRollupWidget = this;
     NS_ADDREF(this);
+	
+		/* Realize the gMenuRegion to capture events outside of the application's canvas when menus are displayed */
+		/* Menus+submenus have the same parent. If the parent has mLastMenu set and realized, then use its region as "infront" */
+		/* If not, then this widget ( mWidget ) becomes the mLastMenu and gets recorded into the parent */
+		/* Different windows have different mLastMenu's */
+		if( mWindowType == eWindowType_popup && !( PtWidgetFlags( gMenuRegion ) & Pt_REALIZED ) ) {
+
+			PtWidget_t *pw = ((nsWindow*)mParent)->mLastMenu;
+
+			if( pw && ( PtWidgetFlags( pw ) & Pt_REALIZED ) && PtWidgetRid( pw ) > 0 )
+				PtSetResource( gMenuRegion, Pt_ARG_REGION_INFRONT, PtWidgetRid( pw ), 0 );
+			else {
+				if( !PtWidgetIsRealized( mWidget ) ) PtRealizeWidget( mWidget );
+				PtSetResource( gMenuRegion, Pt_ARG_REGION_INFRONT, PtWidgetRid( mWidget ), 0 );
+				((nsWindow*)mParent)->mLastMenu = mWidget;
+				}
+
+			PtRealizeWidget( gMenuRegion );
+			}
   	}
 	else {
     NS_IF_RELEASE(gRollupListener);
     gRollupListener = nsnull;
     NS_IF_RELEASE(gRollupWidget);
 		gRollupWidget = nsnull;
+
+		if( mWindowType == eWindowType_popup && ( PtWidgetFlags( gMenuRegion ) & Pt_REALIZED ) )
+			PtUnrealizeWidget( gMenuRegion );
   	}
   
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWindow::Invalidate(PRBool aIsSynchronous)
-{
-	if (!mWidget) return NS_OK; // mWidget will be null during printing.
-	if (!PtWidgetIsRealized(mWidget)) return NS_OK;
-	nsWidget::Invalidate(aIsSynchronous);
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsWindow::Invalidate(const nsRect &aRect, PRBool aIsSynchronous)
-{
-	if (!mWidget) return NS_OK; // mWidget will be null during printing.
-	if (!PtWidgetIsRealized(mWidget)) return NS_OK;
-	nsWidget::Invalidate(aRect, aIsSynchronous);
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsWindow::InvalidateRegion(const nsIRegion* aRegion, PRBool aIsSynchronous)
-{
-	if (!mWidget) return NS_OK; // mWidget will be null during printing.
-	if (!PtWidgetIsRealized(mWidget)) return NS_OK;
-	nsWidget::InvalidateRegion(aRegion,aIsSynchronous);
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsWindow::SetBackgroundColor( const nscolor &aColor ) 
-{
-	return nsWidget::SetBackgroundColor(aColor);
-}
-
-
-NS_IMETHODIMP nsWindow::SetFocus( PRBool aRaise ) {
-  return nsWidget::SetFocus(aRaise);
-	}
-
-	
 NS_METHOD nsWindow::PreCreateWidget( nsWidgetInitData *aInitData ) {
   if (nsnull != aInitData) {
     SetWindowType( aInitData->mWindowType );
     SetBorderStyle( aInitData->mBorderStyle );
     return NS_OK;
   	}
-
   return NS_ERROR_FAILURE;
 	}
 
@@ -251,7 +230,7 @@ NS_METHOD nsWindow::PreCreateWidget( nsWidgetInitData *aInitData ) {
 //
 //-------------------------------------------------------------------------
 NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
-  PtArg_t   arg[20];
+  PtArg_t   arg[25];
   int       arg_count = 0;
   PhPoint_t pos;
   PhDim_t   dim;
@@ -269,14 +248,15 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
     mIsToplevel = PR_TRUE;
     break;
   case eWindowType_toplevel :
+  case eWindowType_invisible :
     mIsToplevel = PR_TRUE;
     break;
   case eWindowType_dialog :
     mIsToplevel = PR_TRUE;
-		break;
+    break;
   case eWindowType_child :
     mIsToplevel = PR_FALSE;
-		break;
+    break;
   }
 	
   if ( mWindowType == eWindowType_child )
@@ -294,7 +274,7 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
   else
   {
     // No border or decorations is the default
-    render_flags = Ph_WM_RENDER_INLINE;
+    render_flags = 0; // Ph_WM_RENDER_RESIZE;
 
     if( mWindowType != eWindowType_popup ) {
 
@@ -332,9 +312,9 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
 	}
     else if ((mWindowType == eWindowType_toplevel) && parentWidget)
     {
-      PhPoint_t p = nsToolkit::GetConsoleOffset();
-      pos.x += p.x;
-      pos.y += p.y;
+			QueryVisible( );
+      pos.x += gConsoleRect.ul.x;
+      pos.y += gConsoleRect.ul.y;
     	PtSetArg( &arg[arg_count++], Pt_ARG_POS, &pos, 0 );
     }
 
@@ -347,15 +327,32 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
       PtSetParentWidget( nsnull );
 		
     if( mWindowType == eWindowType_popup ) {
+
+			/* the gMenuRegion is used to capture events outside of the application's canvas when menus are displayed */
+			if( !gMenuRegion ) {
+				PtArg_t args[10];
+				PtSetParentWidget( NULL );
+				PtSetArg( &args[0], Pt_ARG_REGION_PARENT, 0, 0 );
+	  		PtSetArg( &args[1], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0);
+				PhArea_t area = { { 0, 0 }, { SHRT_MAX, SHRT_MAX } };
+				PtSetArg( &args[2], Pt_ARG_AREA, &area, 0 );
+				PtSetArg( &args[3], Pt_ARG_REGION_SENSE, Ph_EV_BUT_PRESS, Ph_EV_BUT_PRESS );
+				PtSetArg( &args[4], Pt_ARG_REGION_FLAGS, Ph_FORCE_BOUNDARY, Ph_FORCE_BOUNDARY );
+				PtSetArg( &args[5], Pt_ARG_CURSOR_TYPE, Ph_CURSOR_POINTER, 0 );
+				gMenuRegion = PtCreateWidget( PtRegion, NULL, 6, args );
+				PtAddEventHandler( gMenuRegion, Ph_EV_BUT_PRESS, MenuRegionCallback, this );
+				}
+
 	  	int	fields = Ph_REGION_PARENT|Ph_REGION_HANDLE| Ph_REGION_FLAGS|Ph_REGION_ORIGIN|Ph_REGION_EV_SENSE|Ph_REGION_EV_OPAQUE|Ph_REGION_RECT;
 	  	int sense =  Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE | Ph_EV_BUT_REPEAT;
+			PtCallback_t cb_destroyed = { MenuRegionDestroyed, NULL };
 
       PtSetArg( &arg[arg_count++], Pt_ARG_REGION_FIELDS,   fields, fields );
       PtSetArg( &arg[arg_count++], Pt_ARG_REGION_PARENT,   Ph_ROOT_RID, 0 );
       PtSetArg( &arg[arg_count++], Pt_ARG_REGION_SENSE,    sense | Ph_EV_DRAG|Ph_EV_EXPOSE, sense | Ph_EV_DRAG|Ph_EV_EXPOSE);
-      PtSetArg( &arg[arg_count++], Pt_ARG_REGION_OPAQUE,   sense | Ph_EV_DRAG|Ph_EV_EXPOSE|Ph_EV_DRAW, sense |Ph_EV_DRAG|Ph_EV_EXPOSE|Ph_EV_DRAW);
+      PtSetArg( &arg[arg_count++], Pt_ARG_REGION_OPAQUE,   sense | Ph_EV_DRAG|Ph_EV_EXPOSE|Ph_EV_DRAW|Ph_EV_BLIT, sense |Ph_EV_DRAG|Ph_EV_EXPOSE|Ph_EV_DRAW|Ph_EV_BLIT);
       PtSetArg( &arg[arg_count++], Pt_ARG_FLAGS, Pt_DELAY_REALIZE, Pt_GETS_FOCUS | Pt_DELAY_REALIZE);
-	  PtSetArg( &arg[arg_count++], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0);
+			PtSetArg( &arg[arg_count++], Pt_CB_DESTROYED, &cb_destroyed, 0 );
       mWidget = PtCreateWidget( PtRegion, parentWidget, arg_count, arg);
 
     	// Must also create the client-area widget
@@ -363,7 +360,6 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
       PtSetArg( &arg[arg_count++], Pt_ARG_ANCHOR_FLAGS, Pt_ANCHOR_ALL, ~0 );
       PtSetArg( &arg[arg_count++], Pt_ARG_BORDER_WIDTH, 0 , 0 );
       PtSetArg( &arg[arg_count++], Pt_ARG_MARGIN_WIDTH, 0 , 0 );
-       PtSetArg( &arg[arg_count++], Pt_ARG_FILL_COLOR, Pg_GREEN, 0 );
       PhRect_t anch_offset = {{0, 0},{0, 0}};
       PtSetArg( &arg[arg_count++], Pt_ARG_ANCHOR_OFFSETS, &anch_offset, 0 );
 
@@ -372,19 +368,22 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
       mClientWidget = PtCreateWidget( PtRawDrawContainer, mWidget, arg_count, arg );
      }
 	else {
-		PtSetParentWidget( NULL );
-
 		/* Dialog and TopLevel Windows */
 		PtSetArg( &arg[arg_count++], Pt_ARG_FLAGS, Pt_DELAY_REALIZE, Pt_DELAY_REALIZE);
 		PtSetArg( &arg[arg_count++], Pt_ARG_WINDOW_RENDER_FLAGS, render_flags, -1 );
+		PtSetArg( &arg[arg_count++], Pt_ARG_WINDOW_MANAGED_FLAGS, 0, Ph_WM_CLOSE );
+		PtSetArg( &arg[arg_count++], Pt_ARG_WINDOW_NOTIFY_FLAGS, Ph_WM_CLOSE|Ph_WM_CONSWITCH|Ph_WM_FOCUS, ~0 );
 		PtSetArg( &arg[arg_count++], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );
 
 		PtRawCallback_t cb_raw = { Ph_EV_INFO, EvInfo, NULL };
 		PtCallback_t cb_resize = { ResizeHandler, NULL };
+		PtCallback_t cb_window = { WindowWMHandler, this };
+
 		PtSetArg( &arg[arg_count++], Pt_CB_RESIZE, &cb_resize, NULL );
 		PtSetArg( &arg[arg_count++], Pt_CB_RAW, &cb_raw, NULL );
-		mWidget = PtCreateWidget( PtWindow, NULL, arg_count, arg );
-	}
+		PtSetArg( &arg[arg_count++], Pt_CB_WINDOW, &cb_window, 0 );
+		mWidget = PtCreateWidget( PtWindow, parentWidget, arg_count, arg );
+		}
   }
 
   if( mWidget ) {
@@ -394,7 +393,7 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
       	PtAddCallback(mWidget, Pt_CB_RESIZE, ResizeHandler, nsnull ); 
       	PtAddEventHandler( mWidget,
       	  Ph_EV_PTR_MOTION_BUTTON | Ph_EV_PTR_MOTION_NOBUTTON |
-      	  Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE |Ph_EV_BOUNDARY|Ph_EV_DRAG /* | Ph_EV_WM | Ph_EV_EXPOSE */
+      	  Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE |Ph_EV_BOUNDARY|Ph_EV_DRAG
       	  , RawEventHandler, this );
 	
     		PtArg_t arg;
@@ -409,9 +408,11 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
     else if( mWindowType == eWindowType_popup ) {
       		PtAddEventHandler( mClientWidget,
       			 	Ph_EV_PTR_MOTION_BUTTON | Ph_EV_PTR_MOTION_NOBUTTON | 
-      		  	Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE |Ph_EV_BOUNDARY | Ph_EV_WM /*| Ph_EV_EXPOSE*/,
+      		  	Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE |Ph_EV_BOUNDARY,
       			  RawEventHandler, this );
 
+      		PtAddEventHandler( mWidget, Ph_EV_DRAG, RawEventHandler, this );
+
       		PtArg_t arg;
       		PtRawCallback_t callback;
 		
@@ -425,26 +426,7 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
 					}
     else if( !parentWidget ) {
        if( mClientWidget ) PtAddCallback(mClientWidget, Pt_CB_RESIZE, ResizeHandler, nsnull );
-       else {
-#if 0
-		   PtRawCallback_t filter_cb;
-		   
-		   filter_cb.event_mask =  Ph_EV_EXPOSE ;
-		   filter_cb.event_f = RawEventHandler;
-		   filter_cb.data = this;
-		   PtSetResource(mWidget, Pt_CB_FILTER, &filter_cb,1);
-#endif		  
-          	PtAddCallback(mWidget, Pt_CB_RESIZE, ResizeHandler, nsnull ); 
-
-           	PtCallback_t callback;
-		    		callback.event_f = WindowWMHandler;
-            callback.data = this;
-
-            PtSetArg( &arg[0], Pt_CB_WINDOW, &callback, 0 );
-		   PtSetArg( &arg[1], Pt_ARG_WINDOW_NOTIFY_FLAGS, 0xFFFFFFFF, 0XFFFFFFFF );
-            PtSetResources( mWidget, 2, arg );
-       			}
-    		}
+    	}
 
     // call the event callback to notify about creation
     DispatchStandardEvent( NS_CREATE );
@@ -452,6 +434,9 @@ NS_METHOD nsWindow::CreateNative( PtWidget_t *parentWidget ) {
     result = NS_OK;
   	}
 
+  /* force SetCursor to actually set the cursor, even though our internal state indicates that we already
+     have the standard cursor */
+  mCursor = eCursor_wait;
   SetCursor( mCursor );
 
   return result;
@@ -478,17 +463,6 @@ void *nsWindow::GetNativeData(PRUint32 aDataType)
   return nsWidget::GetNativeData(aDataType);
 }
 
-//-------------------------------------------------------------------------
-//
-// Set the colormap of the window
-//
-//-------------------------------------------------------------------------
-NS_METHOD nsWindow::SetColorMap(nsColorMap *aColorMap)
-{
-  return NS_OK;
-}
-
-
 //-------------------------------------------------------------------------
 //
 // Scroll the bits of a window
@@ -572,149 +546,60 @@ NS_METHOD nsWindow::ScrollWidgets( PRInt32 aDx, PRInt32 aDy ) {
 	return NS_OK;    
 	}
 
-NS_IMETHODIMP nsWindow::ScrollRect( nsRect &aSrcRect, PRInt32 aDx, PRInt32 aDy ) {
-	NS_WARNING("nsWindow::ScrollRect Not Implemented\n");
-	return NS_OK;
-	}
-
 NS_METHOD nsWindow::SetTitle( const nsString& aTitle ) {
-  nsresult res = NS_ERROR_FAILURE;
-  char * title = ToNewUTF8String(aTitle);
-  
   if( mWidget ) {
+  	char * title = ToNewUTF8String(aTitle);
     PtSetResource( mWidget, Pt_ARG_WINDOW_TITLE, title, 0 );
-		res = NS_OK;
-  	}
-
-  if (title) nsCRT::free(title);
-
-  return res;
-}
-
-
-/**
- * Processes an Expose Event
- *
- **/
-PRBool nsWindow::OnPaint(nsPaintEvent &event)
-{
+  	if (title) nsCRT::free(title);
+		}
   return NS_OK;
 }
 
 
-NS_METHOD nsWindow::BeginResizingChildren(void)
-{
-	PtHold();
-	return NS_OK;
-}
-
-NS_METHOD nsWindow::EndResizingChildren(void)
-{
-	PtRelease();
-	return NS_OK;
-}
-
-
-
-PRBool nsWindow::OnKey(nsKeyEvent &aEvent)
-{
-  if( mEventCallback ) return DispatchWindowEvent(&aEvent);
-  return PR_FALSE;
-}
-
-
-// nsWindow::DispatchFocus - Not Implemented
-PRBool nsWindow::DispatchFocus(nsGUIEvent &aEvent)
-{
-  return PR_FALSE;
-}
-
-
-// nsWindow::OnScroll - Not Implemented
-PRBool nsWindow::OnScroll(nsScrollbarEvent &aEvent, PRUint32 cPos)
-{
-  return PR_FALSE;
-}
-
-
 NS_IMETHODIMP nsWindow::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepaint)
 {
 	PRBool nNeedToShow = PR_FALSE;
 	
-	if ( aWidth == mBounds.width && aHeight == mBounds.height)
-		return NS_OK;
+	if( aWidth == mBounds.width && aHeight == mBounds.height ) return NS_OK;
 	
 	mBounds.width  = aWidth;
 	mBounds.height = aHeight;
 	
 	// code to keep the window from showing before it has been moved or resized
-	// if we are resized to 1x1 or less, we will hide the window.  Show(TRUE) will be ignored until a
-	// larger resize has happened
-	if (aWidth <= 1 || aHeight <= 1)
-	{
-		if ( mIsToplevel )
-		{
-			aWidth = 1;
-			aHeight = 1;
-			mIsTooSmall = PR_TRUE;
-//			if( mShown ) Show(PR_FALSE);
+	// if we are resized to 1x1 or less, we will hide the window.  Show(TRUE) will be ignored until a larger resize has happened
+	if( aWidth <= 1 || aHeight <= 1 ) {
+		aWidth = aHeight = 1;
+		mIsTooSmall = PR_TRUE;
 		}
-		else
-		{
-			aWidth = 1;
-			aHeight = 1;
-			mIsTooSmall = PR_TRUE;
-//			if( mShown ) Show(PR_FALSE);
-		}
-	}
-	else
-	{
-		if (mIsTooSmall)
-		{
+	else {
+		if( mIsTooSmall ) {
 			// if we are not shown, we don't want to force a show here, so check and see if Show(TRUE) has been called
 			nNeedToShow = mShown;
 			mIsTooSmall = PR_FALSE;
+			}
 		}
-	}
 	
-	PtArg_t  arg;
 	PhDim_t  dim = { aWidth, aHeight };
 	
 	if( mWidget ) {
-		//		EnableDamage( mWidget, PR_FALSE );
-		
 		// center the dialog
 		if( mWindowType == eWindowType_dialog ) {
-			PhPoint_t p, pos = nsToolkit::GetConsoleOffset( );
-			PtCalcAbsPosition(NULL, NULL, &dim, &p);
-			p.x -= pos.x;
-			p.y -= pos.y;
+			PhPoint_t p;
+			QueryVisible( );
+			PtCalcAbsPosition( NULL, NULL, &dim, &p );
+			p.x -= gConsoleRect.ul.x;
+			p.y -= gConsoleRect.ul.y;
 			Move(p.x, p.y); // the move should be in coordinates assuming the console is 0, 0
-		}
-		if ( aRepaint == PR_FALSE )
-			PtStartFlux(mWidget);
-		PtSetArg( &arg, Pt_ARG_DIM, &dim, 0 );
-		PtSetResources( mWidget, 1, &arg );
-		if ( aRepaint == PR_FALSE )
-			PtEndFlux(mWidget);
-		
-		/* This fixes XUL dialogs */
-#if 0
-		if (mClientWidget) {
-			printf("resize!!!\n");
-			PtWidget_t *theClientChild = PtWidgetChildBack(mClientWidget);
-			if (theClientChild) {
-				nsWindow * pWin = (nsWindow*) GetInstance( theClientChild );
-				pWin->Resize(aWidth, aHeight, aRepaint);
 			}
+		if( aRepaint == PR_FALSE )  PtStartFlux(mWidget);
+		PtSetResource( mWidget, Pt_ARG_DIM, &dim, 0 );
+		if( aRepaint == PR_FALSE ) PtEndFlux(mWidget);
 		}
-#endif	
-	}
-	if (mIsToplevel || mListenForResizes) {
-		nsSizeEvent sevent;
+
+	if( mIsToplevel || mListenForResizes ) {
+		nsSizeEvent sevent(PR_TRUE, 0, nsnull);
 		sevent.message = NS_SIZE;
 		sevent.widget = this;
-		sevent.eventStructType = NS_SIZE_EVENT;
 		
 		sevent.windowSize = new nsRect (0, 0, aWidth, aHeight); 	
 		
@@ -728,36 +613,26 @@ NS_IMETHODIMP nsWindow::Resize(PRInt32 aWidth, PRInt32 aHeight, PRBool aRepaint)
 		DispatchWindowEvent(&sevent);
 		Release();
 		delete sevent.windowSize;
-	}
+		}
 	if( nNeedToShow ) Show(PR_TRUE);
 	return NS_OK;
 }
 
-NS_IMETHODIMP nsWindow::Resize( PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight, PRBool aRepaint ) {
-  Move(aX,aY);
-  // resize can cause a show to happen, so do this last
-  Resize(aWidth,aHeight,aRepaint);
-  return NS_OK;
-	}
-
-
 int nsWindow::WindowWMHandler( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo ) 
 {
 	PhWindowEvent_t *we = (PhWindowEvent_t *) cbinfo->cbdata;
-	nsWindow * win = (nsWindow*) data;
-	
 	switch( we->event_f ) {
 		case Ph_WM_CLOSE:
 		  {
+				nsWindow * win = (nsWindow*) data;
 			  NS_ADDREF(win);
 			  
 			  // dispatch an "onclose" event. to delete immediately, call win->Destroy()
-			  nsGUIEvent event;
+			  nsGUIEvent event(PR_TRUE, 0, nsnull);
 			  nsEventStatus status;
 			  
 			  event.message = NS_XUL_CLOSE;
 			  event.widget  = win;
-			  event.eventStructType = NS_GUI_EVENT;
 			  
 			  event.time = 0;
 			  event.point.x = 0;
@@ -768,56 +643,37 @@ int nsWindow::WindowWMHandler( PtWidget_t *widget, void *data, PtCallbackInfo_t
 			  NS_RELEASE(win);
 		  }
 		break;
-		
-		default:
-		if( we->event_f == Ph_WM_TOFRONT || ( we->event_f == Ph_WM_FOCUS && we->state_f == Ph_WM_EVSTATE_FOCUS ) )
-		   return Pt_CONTINUE;
 
-		/* destroy any pop up widgets ( menus ) */
-		if( gRollupWidget && gRollupListener ) gRollupListener->Rollup();
+		case Ph_WM_CONSWITCH:
+			gConsoleRectValid = PR_FALSE; /* force a call tp PhWindowQueryVisible() next time, since we might have moved this window into a different console */
+      /* rollup the menus */
+      if( gRollupWidget && gRollupListener ) gRollupListener->Rollup();
+			break;
+
+		case Ph_WM_FOCUS:
+			if( we->event_state == Ph_WM_EVSTATE_FOCUSLOST ) {
+      	/* rollup the menus */
+      	if( gRollupWidget && gRollupListener ) gRollupListener->Rollup();
+
+				if( sFocusWidget ) sFocusWidget->DispatchStandardEvent(NS_DEACTIVATE);
+				}
+			break;
 	}
 	
 	return Pt_CONTINUE;
 }
 
-//-------------------------------------------------------------------------
-//
-// Process all nsWindows messages
-//
-//-------------------------------------------------------------------------
-PRBool nsWindow::HandleEvent( PtWidget_t *widget, PtCallbackInfo_t* aCbInfo ) {
-/* if in the future you add code here, modify nsWidget::HandleEvent() to allow you this call */
-	return nsWidget::HandleEvent( widget, aCbInfo);
-	}
-
 void nsWindow::RawDrawFunc( PtWidget_t * pWidget, PhTile_t * damage )
 {
   nsWindow  * pWin = (nsWindow*) GetInstance( pWidget );
   nsresult    result;
   PhTile_t  * dmg = NULL;
   PRBool      aClipState;
-  nsPaintEvent pev;
+  nsPaintEvent pev(PR_TRUE, 0, nsnull);
   PhRect_t   extent;
 
-#if 0
-  PgSetFillColor(Pg_WHITE);
-  PgSetDrawMode(Pg_DRAWMODE_XOR);
-  PtWidgetExtent(pWidget, &extent);
-  int i;
-  for (i = 0; i < 5 * 2; i++)
-  {
-	  PgDrawRect(&extent, Pg_DRAW_FILL);
-	  PgFlush();
-	  delay(1);
-  }
-  PgSetDrawMode(Pg_DRAWMODE_OPAQUE);
-#endif	
   if( !pWin || !pWin->mContext ) return;
 
-  // This prevents redraws while any window is resizing, ie there are
-  //   windows in the resize queue
-  if( pWin->mIsResizing ) return;
-
 	if ( pWin->mEventCallback ) {
 		PhPoint_t  offset;
 		nsRect     nsDmg;
@@ -831,7 +687,7 @@ void nsWindow::RawDrawFunc( PtWidget_t * pWidget, PhTile_t * damage )
 		/* Intersect the Damage tile list w/ the clipped out list and see whats left! */
 		new_damage = PhRectsToTiles(&damage->rect, 1);
 		PhDeTranslateTiles(new_damage, &offset);
-		clip_tiles = pWin->GetWindowClipping();
+		clip_tiles = GetWindowClipping( pWidget );
 		if (clip_tiles) {
 			new_damage = PhClipTilings( new_damage, clip_tiles, NULL);
 			PhFreeTiles(clip_tiles);
@@ -845,7 +701,7 @@ void nsWindow::RawDrawFunc( PtWidget_t * pWidget, PhTile_t * damage )
 		new_damage = intersect;
 		
 		pWin->InitEvent(pev, NS_PAINT);
-		pev.eventStructType = NS_PAINT_EVENT;
+		pev.region = nsnull;
 		pev.renderingContext = nsnull;
 		pev.renderingContext = pWin->GetRenderingContext();
 		for( dmg = new_damage; dmg; dmg = dmg->next ) {
@@ -859,10 +715,10 @@ void nsWindow::RawDrawFunc( PtWidget_t * pWidget, PhTile_t * damage )
 
 			/* Re-Setup Paint Event */
 			pWin->InitEvent(pev, NS_PAINT);
-			pev.eventStructType = NS_PAINT_EVENT;
 			pev.point.x = nsDmg.x;
 			pev.point.y = nsDmg.y;
 			pev.rect = &nsDmg;
+			pev.region = nsnull;
 
 			if( pev.renderingContext ) {
 				nsIRegion *ClipRegion = pWin->GetRegion( );
@@ -880,23 +736,8 @@ void nsWindow::RawDrawFunc( PtWidget_t * pWidget, PhTile_t * damage )
 	}
 }
 
-
-void nsWindow::ScreenToWidget( PhPoint_t &pt ) {
-  // pt is in screen coordinates
-  // convert it to be relative to ~this~ widgets origin
-  short x,y;
-
-  if( mWindowType == eWindowType_child )
-    PtGetAbsPosition( mWidget, &x, &y );
-  else PtGetAbsPosition( mClientWidget, &x, &y );
-
-  pt.x -= x;
-  pt.y -= y;
-	}
-
-
-PhTile_t *nsWindow::GetWindowClipping( ) {
-	PtWidget_t *w, *aWidget = (PtWidget_t *)GetNativeData(NS_NATIVE_WIDGET);
+static PhTile_t *GetWindowClipping( PtWidget_t *aWidget ) {
+	PtWidget_t *w;
 	PhTile_t *clip_tiles = NULL, *last = NULL;
 	PhRect_t w_extent;
 
@@ -904,7 +745,7 @@ PhTile_t *nsWindow::GetWindowClipping( ) {
 	
 	for( w = PtWidgetChildFront( aWidget ); w; w=PtWidgetBrotherBehind( w ) ) {
 		long flags = PtWidgetFlags( w );
-		if( (flags & (Pt_OPAQUE|Pt_REALIZED)) && !(PtIsDisjoint(w))) {
+		if( (flags & Pt_REALIZED) && (flags & Pt_OPAQUE) && !PtIsDisjoint(w) ) {
 			PhTile_t *tile = PhGetTile( );
 			if( !tile ) return NULL;
 
@@ -924,28 +765,26 @@ PhTile_t *nsWindow::GetWindowClipping( ) {
 
 int nsWindow::ResizeHandler( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
 {
-  PtContainerCallback_t *cb = (PtContainerCallback_t *) cbinfo->cbdata;
-  PhRect_t *extents = &cb->new_size;
   nsWindow *someWindow = (nsWindow *) GetInstance(widget);
-  nsRect rect;
-
-  if( someWindow && extents)	// kedl
-  {
-	rect.x = extents->ul.x;
+  if( someWindow ) {
+  	PtContainerCallback_t *cb = (PtContainerCallback_t *) cbinfo->cbdata;
+  	PhRect_t *extents = &cb->new_size;
+  	nsRect rect;
+		rect.x = extents->ul.x;
     rect.y = extents->ul.y;
     rect.width = extents->lr.x - rect.x + 1;
     rect.height = extents->lr.y - rect.y + 1;
-    if ( someWindow->mBounds.width == rect.width && someWindow->mBounds.height == rect.height)
-		  return (Pt_CONTINUE);
-    someWindow->mBounds.width  = rect.width;	// kedl, fix main window not resizing!!!!
+
+    if( someWindow->mBounds.width == rect.width && someWindow->mBounds.height == rect.height )
+		  return Pt_CONTINUE;
+
+    someWindow->mBounds.width  = rect.width;
     someWindow->mBounds.height = rect.height;
+
     /* This enables the resize holdoff */
-    if (PtWidgetIsRealized(widget)) {
-//      someWindow->ResizeHoldOff();
-      someWindow->OnResize( rect );
-    }
-  }
-	return( Pt_CONTINUE );
+    if( PtWidgetIsRealized( widget ) ) someWindow->OnResize( rect );
+  	}
+	return Pt_CONTINUE;
 }
 
 
@@ -983,8 +822,7 @@ int nsWindow::EvInfo( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
 			nsCOMPtr<nsIPresShell> presShell;
 			docShell->GetPresShell( getter_AddRefs(presShell) );
 
-			nsCOMPtr<nsIViewManager> viewManager;
-			presShell->GetViewManager(getter_AddRefs(viewManager));
+			nsIViewManager* viewManager = presShell->GetViewManager();
 			NS_ENSURE_TRUE(viewManager, NS_ERROR_FAILURE);
 
 			windowEnumerator->HasMoreElements(&more);
@@ -995,134 +833,18 @@ int nsWindow::EvInfo( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
 	return Pt_CONTINUE;
 	}
 
-void nsWindow::ResizeHoldOff() {
-
-  if( !mWidget ) return;
-
-  if( PR_FALSE == mResizeQueueInited ) {
-    // This is to guarantee that the Invalidation work-proc is in place prior to the resize work-proc.
-//    if( !mDmgQueueInited ) Invalidate( PR_FALSE );
-
-    PtWidget_t *top = PtFindDisjoint( mWidget );
-
-		if( (mResizeProcID = PtAppAddWorkProc( nsnull, ResizeWorkProc, top )) != nsnull ) {
-			PtHold();
-			mResizeQueueInited = PR_TRUE;
-			}
-		}
-
-  if( PR_TRUE == mResizeQueueInited ) {
-    DamageQueueEntry *dqe;
-    PRBool           found = PR_FALSE;
-  
-		for( dqe = mResizeQueue; dqe; dqe = dqe->next ) {
-      if( dqe->widget == mWidget ) {
-        found = PR_TRUE;
-        break;
-      	}
-    	}
-
-    if( !found ) {
-      dqe = new DamageQueueEntry;
-      if( dqe ) {
-        mIsResizing = PR_TRUE;
-        dqe->widget = mWidget;
-        dqe->inst = this;
-        dqe->next = mResizeQueue;
-        mResizeQueue = dqe;
-      	}
-    	}
-  	}
-	}
-
-
-void nsWindow::RemoveResizeWidget( ) {
-  if( mIsResizing ) {
-    DamageQueueEntry *dqe;
-    DamageQueueEntry *last_dqe = nsnull;
-  
-    dqe = mResizeQueue;
-
-    // If this widget is in the queue, remove it
-    while( dqe ) {
-      if( dqe->widget == mWidget ) {
-        if( last_dqe ) last_dqe->next = dqe->next;
-        else mResizeQueue = dqe->next;
-
-        delete dqe;
-        mIsResizing = PR_FALSE;
-        break;
-      	}
-      last_dqe = dqe;
-      dqe = dqe->next;
-    	}
-
-    if( nsnull == mResizeQueue ) {
-      mResizeQueueInited = PR_FALSE;
-      PtRelease( );
-      if( mResizeProcID ) PtAppRemoveWorkProc( nsnull, mResizeProcID );
-    	}
-  	}
-	}
-
-
-int nsWindow::ResizeWorkProc( void *data ) {
-
-  if ( mResizeQueueInited ) {
-    DamageQueueEntry *dqe = nsWindow::mResizeQueue;
-    DamageQueueEntry *last_dqe;
-
-    while( dqe ) {
-      ((nsWindow*)dqe->inst)->mIsResizing = PR_FALSE;
-      dqe->inst->Invalidate( PR_FALSE );
-      last_dqe = dqe;
-      dqe = dqe->next;
-      delete last_dqe;
-    	}
-
-    nsWindow::mResizeQueue = nsnull;
-    nsWindow::mResizeQueueInited = PR_FALSE;
-
-    PtRelease();
-  	}
-
-  return Pt_END;
-	}
-
-
-NS_METHOD nsWindow::GetClientBounds( nsRect &aRect ) {
-  aRect.x = 0;
-  aRect.y = 0;
-  aRect.width = mBounds.width;
-  aRect.height = mBounds.height;
-  return NS_OK;
-	}
-
-//-------------------------------------------------------------------------
-//
-// grab mouse events for this widget
-// CaptureMouse Not Implemented
-//-------------------------------------------------------------------------
-NS_IMETHODIMP nsWindow::CaptureMouse( PRBool aCapture ) {
-  return NS_OK;
-	}
-
-NS_METHOD nsWindow::ConstrainPosition( PRBool aAllowSlop,
-                                       PRInt32 *aX, PRInt32 *aY ) {
-  return NS_OK;
-}
-
 //-------------------------------------------------------------------------
 //
 // Move this component
 //
 //-------------------------------------------------------------------------
 NS_METHOD nsWindow::Move( PRInt32 aX, PRInt32 aY ) {
-	PRInt32 origX, origY;
 
-	/* Keep a untouched version of the coordinates laying around for comparison */
-	origX=aX;
-	origY=aY;
+	if( mWindowType != eWindowType_popup && (mBounds.x == aX) && (mBounds.y == aY) )
+		return NS_OK;
+
+	mBounds.x = aX;
+	mBounds.y = aY;
 
 	switch( mWindowType ) {
 		case eWindowType_popup:
@@ -1147,7 +869,6 @@ NS_METHOD nsWindow::Move( PRInt32 aX, PRInt32 aY ) {
 			aX += total_offset.x;
 			aY += total_offset.y;
 
-
 			/* Add the Offset if the widget is offset from its parent.. */
 			PtWidgetOffset( mWidget->parent, &offset );
 			aX += offset.x;
@@ -1158,55 +879,30 @@ NS_METHOD nsWindow::Move( PRInt32 aX, PRInt32 aY ) {
 		case eWindowType_dialog:
 		case eWindowType_toplevel:
 			/* Offset to the current virtual console */
-			PhPoint_t offset = nsToolkit::GetConsoleOffset();
-			aX += offset.x;
-			aY += offset.y;
+			QueryVisible( );
+			aX += gConsoleRect.ul.x;
+			aY += gConsoleRect.ul.y;
 			break;
 		}
 
-	/* Call my base class */
-	nsresult res = nsWidget::Move(aX, aY);
+  if( mWidget ) {
+    if(( mWidget->area.pos.x != aX ) || ( mWidget->area.pos.y != aY )) {
+      PhPoint_t pos = { aX, aY };
+      PtSetResource( mWidget, Pt_ARG_POS, &pos, 0 );
+    	}
+  	}
 
-	/* If I am a top-level window my origin should always be 0,0 */
-	if( mWindowType != eWindowType_child ) {
-		mBounds.x = origX;
-		mBounds.y = origY;
-		}
-
-	return res;
-	}
-
-//-------------------------------------------------------------------------
-//
-// Determine whether an event should be processed, assuming we're
-// a modal window.
-//
-//-------------------------------------------------------------------------
-NS_METHOD nsWindow::ModalEventFilter(PRBool aRealEvent, void *aEvent, PRBool *aForWindow) {
-	*aForWindow = PR_TRUE;
 	return NS_OK;
 	}
 
 int nsWindow::MenuRegionCallback( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo ) {
-
-	if( cbinfo->event->type == Ph_EV_BUT_PRESS ) {
-  	nsWindow      *pWin = (nsWindow*) data;
-		if( pWin->gRollupWidget && pWin->gRollupListener ) {
-			/* Close the Window */
-			pWin->gRollupListener->Rollup();
-			}
+	if( gRollupWidget && gRollupListener ) {
+		/* rollup the menu */
+		gRollupListener->Rollup();
 		}
 	return Pt_CONTINUE;
 	}
 
-NS_METHOD nsWindow::GetAttention( ) {
-  if( mWidget && mWidget->parent && !PtIsFocused( mWidget ) ) {
-    /* Raise to the top */
-    PtWidgetToFront(mWidget);  
-  	}
-  return NS_OK;
-	}
-
 NS_IMETHODIMP nsWindow::SetModal( PRBool aModal ) {
   nsresult res = NS_ERROR_FAILURE;
  
@@ -1229,7 +925,7 @@ NS_IMETHODIMP nsWindow::SetModal( PRBool aModal ) {
 	}
 
 
-nsIRegion *nsWindow::GetRegion()
+inline nsIRegion *nsWindow::GetRegion()
 {
   nsIRegion *region = NULL;
   nsresult res;
@@ -1243,3 +939,74 @@ nsIRegion *nsWindow::GetRegion()
   
   return region;  
 }
+
+/*
+	widget is a PtRegion representing the native widget for a menu - reset the mParent->mLastMenu
+	since it points to an widget being destroyed
+*/
+int nsWindow::MenuRegionDestroyed( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
+{
+	nsWindow *pWin = (nsWindow *) GetInstance( widget );
+	if( pWin ) {
+		nsWindow *parent = ( nsWindow * ) pWin->mParent;
+		if( parent && parent->mLastMenu == widget )
+			parent->mLastMenu = nsnull;
+		}
+	return Pt_CONTINUE;
+}
+
+NS_IMETHODIMP nsWindow::SetFocus(PRBool aRaise)
+{
+	sFocusWidget = this;
+
+	if( PtIsFocused( mWidget ) == 2 ) return NS_OK;
+
+	if( mWidget ) {
+		PtWidget_t *disjoint;
+		disjoint = PtFindDisjoint( mWidget );
+		if( PtWidgetIsClass( disjoint, PtWindow ) ) {
+			if( !( PtWindowGetState( disjoint ) & Ph_WM_STATE_ISFOCUS ) ) {
+				nsWindow *pWin = (nsWindow *) GetInstance( disjoint );
+				pWin->GetAttention( -1 );
+				}
+			}
+		PtContainerGiveFocus( mWidget, NULL );
+		}
+	return NS_OK;
+}
+
+NS_IMETHODIMP nsWindow::MakeFullScreen(PRBool aFullScreen)
+{
+	/* we can use static data here because there can be only one full-screen window at a time */
+	static unsigned short old_render_flags;
+	static PhPoint_t old_pos;
+
+	PtArg_t args[3];
+
+	if( aFullScreen ) {
+		unsigned short p, *pflags;
+		PhArea_t area;
+
+		PtSetArg( &args[0], Pt_ARG_WINDOW_RENDER_FLAGS, &pflags, 0 );
+		PtGetResources( mWidget, 1, args );
+		p = old_render_flags = *pflags; // save the render flags
+		p &= ~(Ph_WM_RENDER_TITLE|Ph_WM_RENDER_BORDER);
+
+		PtWidgetArea( mWidget, &area );
+		old_pos = area.pos;
+
+		QueryVisible( );
+		PtSetArg( &args[0], Pt_ARG_POS, &gConsoleRect.ul, 0 );
+		PtSetArg( &args[1], Pt_ARG_WINDOW_RENDER_FLAGS, p, -1 );
+		PtSetArg( &args[2], Pt_ARG_WINDOW_STATE, Ph_WM_STATE_ISFRONT, Ph_WM_STATE_ISFRONT );
+		PtSetResources( mWidget, 3, args );
+		}
+	else {
+		PtSetArg( &args[0], Pt_ARG_POS, &old_pos, 0 );
+		PtSetArg( &args[1], Pt_ARG_WINDOW_RENDER_FLAGS, old_render_flags, -1 ); /* restore the render flags to the previous value */
+		PtSetArg( &args[2], Pt_ARG_WINDOW_STATE, Ph_WM_STATE_ISNORMAL, Ph_WM_STATE_ISFRONT|Ph_WM_STATE_ISNORMAL );
+		PtSetResources( mWidget, 3, args );
+		}
+
+	return nsBaseWidget::MakeFullScreen( aFullScreen );
+}
